@@ -39,32 +39,63 @@
 #
 # $Id$
 
-class ControlSlycot(Exception):
+class ControlSlycot(ImportError):
     """Exception for Slycot import.  Used when we can't import a function
     from the slycot package"""
     pass
 
-class ControlDimension(Exception):
+class ControlDimension(ValueError):
     """Raised when dimensions of system objects are not correct"""
     pass
 
-class ControlArgument(Exception):
+class ControlArgument(TypeError):
     """Raised when arguments to a function are not correct"""
     pass
 
-class ControlMIMONotImplemented(Exception):
+class ControlMIMONotImplemented(NotImplementedError):
     """Function is not currently implemented for MIMO systems"""
     pass
 
-class ControlNotImplemented(Exception):
+class ControlNotImplemented(NotImplementedError):
     """Functionality is not yet implemented"""
     pass
 
 # Utility function to see if slycot is installed
+slycot_installed = None
 def slycot_check():
-    try:
-        import slycot
-    except:
-        return False
-    else:
-        return True
+    """Return True if slycot is installed, otherwise False."""
+    global slycot_installed
+    if slycot_installed is None:
+        try:
+            import slycot
+            slycot_installed = True
+        except:
+            slycot_installed = False
+    return slycot_installed
+
+
+# Utility function to see if pandas is installed
+pandas_installed = None
+def pandas_check():
+    """Return True if pandas is installed, otherwise False."""
+    global pandas_installed
+    if pandas_installed is None:
+        try:
+            import pandas
+            pandas_installed = True
+        except:
+            pandas_installed = False
+    return pandas_installed
+
+# Utility function to see if cvxopt is installed
+cvxopt_installed = None
+def cvxopt_check():
+    """Return True if cvxopt is installed, otherwise False."""
+    global cvxopt_installed
+    if cvxopt_installed is None:
+        try:
+            import cvxopt
+            cvxopt_installed = True
+        except:
+            cvxopt_installed = False
+    return cvxopt_installed
