@@ -44,12 +44,17 @@ FRD data.
 """
 
 # External function declarations
+from copy import copy
 from warnings import warn
+
 import numpy as np
 from numpy import angle, array, empty, ones, \
     real, imag, absolute, eye, linalg, where, sort
 from scipy.interpolate import splprep, splev
+
 from .lti import LTI, _process_frequency_response
+from .exception import pandas_check
+from .namedio import NamedIOSystem, _process_namedio_keywords
 from . import config
 
 __all__ = ['FrequencyResponseData', 'FRD', 'frd']
@@ -97,7 +102,7 @@ class FrequencyResponseData(LTI):
     second dimension corresponding to the input index, and the 3rd dimension
     corresponding to the frequency points in omega.  For example,
 
-    >>> frdata[2,5,:] = numpy.array([1., 0.8-0.2j, 0.2-0.8j])
+    >>> frdata[2,5,:] = numpy.array([1., 0.8-0.2j, 0.2-0.8j])   # doctest: +SKIP
 
     means that the frequency response from the 6th input to the 3rd output at
     the frequencies defined in omega is set to the array above, i.e. the rows
@@ -112,7 +117,7 @@ class FrequencyResponseData(LTI):
 
     # Allow NDarray * StateSpace to give StateSpace._rmul_() priority
     # https://docs.scipy.org/doc/numpy/reference/arrays.classes.html
-    __array_priority__ = 11     # override ndarray and matrix types
+    __array_priority__ = 13     # override ndarray, StateSpace, I/O sys
 
     #
     # Class attributes
@@ -139,19 +144,22 @@ class FrequencyResponseData(LTI):
         The default constructor is FRD(d, w), where w is an iterable of
         frequency points, and d is the matching frequency data.
 
-        If d is a single list, 1d array, or tuple, a SISO system description
+        If d is a single list, 1D array, or tuple, a SISO system description
         is assumed. d can also be
 
         To call the copy constructor, call FRD(sys), where sys is a
         FRD object.
 
         To construct frequency response data for an existing LTI
-        object, other than an FRD, call FRD(sys, omega)
+        object, other than an FRD, call FRD(sys, omega).
 
         """
         # TODO: discrete-time FRD systems?
-        smooth = kwargs.get('smooth', False)
+        smooth = kwargs.pop('smooth', False)
 
+        #
+        # Process positional arguments
+        #
         if len(args) == 2:
             if not isinstance(args[0], FRD) and isinstance(args[0], LTI):
                 # not an FRD, but still a system, second argument should be
@@ -168,13 +176,12 @@ class FrequencyResponseData(LTI):
 
             else:
                 # The user provided a response and a freq vector
-                self.fresp = array(args[0], dtype=complex)
-                if len(self.fresp.shape) == 1:
-                    self.fresp = self.fresp.reshape(1, 1, len(args[0]))
-                self.omega = array(args[1], dtype=float)
-                if len(self.fresp.shape) != 3 or \
-                        self.fresp.shape[-1] != self.omega.shape[-1] or \
-                        len(self.omega.shape) != 1:
+                self.fresp = array(args[0], dtype=complex, ndmin=1)
+                if self.fresp.ndim == 1:
+                    self.fresp = self.fresp.reshape(1, 1, -1)
+                self.omega = array(args[1], dtype=float, ndmin=1)
+                if self.fresp.ndim != 3 or self.omega.ndim != 1 or \
+                        self.fresp.shape[-1] != self.omega.shape[-1]:
                     raise TypeError(
                         "The frequency data constructor needs a 1-d or 3-d"
                         " response data array and a matching frequency vector"
@@ -192,6 +199,29 @@ class FrequencyResponseData(LTI):
             raise ValueError(
                 "Needs 1 or 2 arguments; received %i." % len(args))
 
+        #
+        # Process key word arguments
+        #
+        # Keep track of return type
+        self.return_magphase=kwargs.pop('return_magphase', False)
+        if self.return_magphase not in (True, False):
+            raise ValueError("unknown return_magphase value")
+
+        # Determine whether to squeeze the output
+        self.squeeze=kwargs.pop('squeeze', None)
+        if self.squeeze not in (None, True, False):
+            raise ValueError("unknown squeeze value")
+
+        # Process namedio keywords
+        defaults = {
+            'inputs': self.fresp.shape[1], 'outputs': self.fresp.shape[0]}
+        name, inputs, outputs, states, dt = _process_namedio_keywords(
+                kwargs, defaults, end=True)
+
+        # Process signal names
+        NamedIOSystem.__init__(
+            self, name=name, inputs=inputs, outputs=outputs, dt=dt)
+
         # create interpolation functions
         if smooth:
             self.ifunc = empty((self.fresp.shape[0], self.fresp.shape[1]),
@@ -204,7 +234,29 @@ class FrequencyResponseData(LTI):
                         w=1.0/(absolute(self.fresp[i, j, :]) + 0.001), s=0.0)
         else:
             self.ifunc = None
-        LTI.__init__(self, self.fresp.shape[1], self.fresp.shape[0])
+
+    #
+    # Frequency response properties
+    #
+    # Different properties of the frequency response that can be used for
+    # analysis and characterization.
+    #
+
+    @property
+    def magnitude(self):
+        return np.abs(self.fresp)
+
+    @property
+    def phase(self):
+        return np.angle(self.fresp)
+
+    @property
+    def frequency(self):
+        return self.omega
+
+    @property
+    def response(self):
+        return self.fresp
 
     def __str__(self):
         """String representation of the transfer function."""
@@ -256,11 +308,13 @@ class FrequencyResponseData(LTI):
 
         # Check that the input-output sizes are consistent.
         if self.ninputs != other.ninputs:
-            raise ValueError("The first summand has %i input(s), but the \
-second has %i." % (self.ninputs, other.ninputs))
+            raise ValueError(
+                "The first summand has %i input(s), but the " \
+                "second has %i." % (self.ninputs, other.ninputs))
         if self.noutputs != other.noutputs:
-            raise ValueError("The first summand has %i output(s), but the \
-second has %i." % (self.noutputs, other.noutputs))
+            raise ValueError(
+                "The first summand has %i output(s), but the " \
+                "second has %i." % (self.noutputs, other.noutputs))
 
         return FRD(self.fresp + other.fresp, other.omega)
 
@@ -354,10 +408,6 @@ second has %i." % (self.noutputs, other.noutputs))
                    smooth=(self.ifunc is not None) and
                           (other.ifunc is not None))
 
-    # TODO: Remove when transition to python3 complete
-    def __div__(self, other):
-        return self.__truediv__(other)
-
     # TODO: Division of MIMO transfer function objects is not written yet.
     def __rtruediv__(self, other):
         """Right divide two LTI objects."""
@@ -375,10 +425,6 @@ second has %i." % (self.noutputs, other.noutputs))
 
         return other / self
 
-    # TODO: Remove when transition to python3 complete
-    def __rdiv__(self, other):
-        return self.__rtruediv__(other)
-
     def __pow__(self, other):
         if not type(other) == int:
             raise ValueError("Exponent must be an integer")
@@ -456,7 +502,7 @@ second has %i." % (self.noutputs, other.noutputs))
 
         return _process_frequency_response(self, omega, out, squeeze=squeeze)
 
-    def __call__(self, s, squeeze=None):
+    def __call__(self, s=None, squeeze=None, return_magphase=None):
         """Evaluate system's transfer function at complex frequencies.
 
         Returns the complex frequency response `sys(s)` of system `sys` with
@@ -469,17 +515,31 @@ second has %i." % (self.noutputs, other.noutputs))
         For a frequency response data object, the argument must be an
         imaginary number (since only the frequency response is defined).
 
+        If ``s`` is not given, this function creates a copy of a frequency
+        response data object with a different set of output settings.
+
         Parameters
         ----------
         s : complex scalar or 1D array_like
-            Complex frequencies
-        squeeze : bool, optional (default=True)
+            Complex frequencies.  If not specified, return a copy of the
+            frequency response data object with updated settings for output
+            processing (``squeeze``, ``return_magphase``).
+
+        squeeze : bool, optional
             If squeeze=True, remove single-dimensional entries from the shape
             of the output even if the system is not SISO. If squeeze=False,
             keep all indices (output, input and, if omega is array_like,
             frequency) even if the system is SISO. The default value can be
             set using config.defaults['control.squeeze_frequency_response'].
 
+        return_magphase : bool, optional
+            If True, then a frequency response data object will enumerate as a
+            tuple of the form (mag, phase, omega) where where ``mag`` is the
+            magnitude (absolute value, not dB or log10) of the system
+            frequency response, ``phase`` is the wrapped phase in radians of
+            the system frequency response, and ``omega`` is the (sorted)
+            frequencies at which the response was evaluated.
+
         Returns
         -------
         fresp : complex ndarray
@@ -498,6 +558,17 @@ second has %i." % (self.noutputs, other.noutputs))
             frequency values.
 
         """
+        if s is None:
+            # Create a copy of the response with new keywords
+            response = copy(self)
+
+            # Update any keywords that we were passed
+            response.squeeze = self.squeeze if squeeze is None else squeeze
+            response.return_magphase = self.return_magphase \
+                if return_magphase is None else return_magphase
+
+            return response
+
         # Make sure that we are operating on a simple list
         if len(np.atleast_1d(s).shape) > 1:
             raise ValueError("input list must be 1D")
@@ -512,6 +583,22 @@ second has %i." % (self.noutputs, other.noutputs))
         else:
             return self.eval(complex(s).imag, squeeze=squeeze)
 
+    # Implement iter to allow assigning to a tuple
+    def __iter__(self):
+        fresp = _process_frequency_response(
+            self, self.omega, self.fresp, squeeze=self.squeeze)
+        if not self.return_magphase:
+            return iter((self.omega, fresp))
+        return iter((np.abs(fresp), np.angle(fresp), self.omega))
+
+    # Implement (thin) getitem to allow access via legacy indexing
+    def __getitem__(self, index):
+        return list(self.__iter__())[index]
+
+    # Implement (thin) len to emulate legacy testing interface
+    def __len__(self):
+        return 3 if self.return_magphase else 2
+
     def freqresp(self, omega):
         """(deprecated) Evaluate transfer function at complex frequencies.
 
@@ -547,6 +634,22 @@ second has %i." % (self.noutputs, other.noutputs))
 
         return FRD(fresp, other.omega, smooth=(self.ifunc is not None))
 
+    # Convert to pandas
+    def to_pandas(self):
+        if not pandas_check():
+            ImportError('pandas not installed')
+        import pandas
+
+        # Create a dict for setting up the data frame
+        data = {'omega': self.omega}
+        data.update(
+            {'H_{%s, %s}' % (out, inp): self.fresp[i, j] \
+             for i, out in enumerate(self.output_labels) \
+             for j, inp in enumerate(self.input_labels)})
+
+        return pandas.DataFrame(data)
+
+
 #
 # Allow FRD as an alias for the FrequencyResponseData class
 #
@@ -557,8 +660,6 @@ second has %i." % (self.noutputs, other.noutputs))
 # FrequenceResponseData and then assigning FRD to point to the same object
 # fixes this problem.
 #
-
-
 FRD = FrequencyResponseData
 
 
@@ -572,8 +673,17 @@ def _convert_to_FRD(sys, omega, inputs=1, outputs=1):
     scalar, then the number of inputs and outputs can be specified
     manually, as in:
 
+    >>> import numpy as np
+    >>> from control.frdata import _convert_to_FRD
+
+    >>> omega = np.logspace(-1, 1)
     >>> frd = _convert_to_FRD(3., omega) # Assumes inputs = outputs = 1
-    >>> frd = _convert_to_FRD(1., omegs, inputs=3, outputs=2)
+    >>> frd.ninputs, frd.noutputs
+    (1, 1)
+
+    >>> frd = _convert_to_FRD(1., omega, inputs=3, outputs=2)
+    >>> frd.ninputs, frd.noutputs
+    (3, 2)
 
     In the latter example, sys's matrix transfer function is [[1., 1., 1.]
                                                               [1., 1., 1.]].
@@ -654,5 +764,17 @@ def frd(*args):
     See Also
     --------
     FRD, ss, tf
+
+    Examples
+    --------
+    >>> # Create from measurements
+    >>> response = [1.0, 1.0, 0.5]
+    >>> freqs = [1, 10, 100]
+    >>> F = ct.frd(response, freqs)
+
+    >>> G = ct.tf([1], [1, 1])
+    >>> freqs = [1, 10, 100]
+    >>> F = ct.frd(G, freqs)
+
     """
     return FRD(*args)
