@@ -62,8 +62,10 @@ if not ('.config' in sys.modules):
 
 # Control system library
 from ..statesp import *
+from ..iosys import ss, rss, drss       # moved from .statesp
 from ..xferfcn import *
 from ..lti import *
+from ..namedio import *
 from ..frdata import *
 from ..dtime import *
 from ..exception import ControlArgument
@@ -82,6 +84,10 @@ from ..margins import margin
 from ..rlocus import rlocus
 from ..dtime import c2d
 from ..sisotool import sisotool
+from ..stochsys import lqe, dlqe
+
+# Functions that are renamed in MATLAB
+pole, zero = poles, zeros
 
 # Import functions specific to Matlab compatibility package
 from .timeresp import *
@@ -109,7 +115,7 @@ Creating linear models
 
 ==  ==========================  ============================================
 \*  :func:`tf`                  create transfer function (TF) models
-\   zpk                         create zero/pole/gain (ZPK) models.
+\*  :func:`zpk`                 create zero/pole/gain (ZPK) models.
 \*  :func:`ss`                  create state-space (SS) models
 \   dss                         create descriptor state-space models
 \   delayss                     create state-space models with delayed terms
@@ -183,7 +189,7 @@ System gain and dynamics
 
 ==  ==========================  ============================================
 \*  :func:`dcgain`              steady-state (D.C.) gain
-\   lti/bandwidth               system bandwidth
+\*  :func:`bandwidth`           system bandwidth
 \   lti/norm                    h2 and Hinfinity norms of LTI models
 \*  :func:`pole`                system poles
 \*  :func:`zero`                system (transmission) zeros
