@@ -3,9 +3,9 @@ Wrappers for the MATLAB compatibility module
 """
 
 import numpy as np
-from ..statesp import ss
+from ..iosys import ss
 from ..xferfcn import tf
-from ..ctrlutil import issys
+from ..lti import LTI
 from ..exception import ControlArgument
 from scipy.signal import zpk2tf
 from warnings import warn
@@ -26,11 +26,13 @@ def bode(*args, **kwargs):
         a list of systems can be entered, or several systems can be
         specified (i.e. several parameters). The sys arguments may also be
         interspersed with format strings. A frequency argument (array_like)
-        may also be added, some examples:
-        * >>> bode(sys, w)                    # one system, freq vector
-        * >>> bode(sys1, sys2, ..., sysN)     # several systems
-        * >>> bode(sys1, sys2, ..., sysN, w)
-        * >>> bode(sys1, 'plotstyle1', ..., sysN, 'plotstyleN') # + plot formats
+        may also be added, some examples::
+
+        >>> bode(sys, w)                    # one system, freq vector              # doctest: +SKIP
+        >>> bode(sys1, sys2, ..., sysN)     # several systems                      # doctest: +SKIP
+        >>> bode(sys1, sys2, ..., sysN, w)                                         # doctest: +SKIP
+        >>> bode(sys1, 'plotstyle1', ..., sysN, 'plotstyleN') # + plot formats     # doctest: +SKIP
+
     omega: freq_range
         Range of frequencies in rad/s
     dB : boolean
@@ -44,6 +46,8 @@ def bode(*args, **kwargs):
 
     Examples
     --------
+    >>> from control.matlab import ss, bode
+
     >>> sys = ss("1. -2; 3. -4", "5.; 7", "6. 8", "9.")
     >>> mag, phase, omega = bode(sys)
 
@@ -51,10 +55,10 @@ def bode(*args, **kwargs):
 
         Document these use cases
 
-        * >>> bode(sys, w)
-        * >>> bode(sys1, sys2, ..., sysN)
-        * >>> bode(sys1, sys2, ..., sysN, w)
-        * >>> bode(sys1, 'plotstyle1', ..., sysN, 'plotstyleN')
+        * >>> bode(sys, w)                                      # doctest: +SKIP
+        * >>> bode(sys1, sys2, ..., sysN)                       # doctest: +SKIP
+        * >>> bode(sys1, sys2, ..., sysN, w)                    # doctest: +SKIP
+        * >>> bode(sys1, 'plotstyle1', ..., sysN, 'plotstyleN') # doctest: +SKIP
     """
     from ..freqplot import bode_plot
 
@@ -117,10 +121,10 @@ def nyquist(*args, **kwargs):
 def _parse_freqplot_args(*args):
     """Parse arguments to frequency plot routines (bode, nyquist)"""
     syslist, plotstyle, omega, other = [], [], None, {}
-    i = 0;
+    i = 0
     while i < len(args):
         # Check to see if this is a system of some sort
-        if issys(args[i]):
+        if isinstance(args[i], LTI):
             # Append the system to our list of systems
             syslist.append(args[i])
             i += 1
@@ -178,8 +182,7 @@ ngrid.__doc__ = nichols_grid.__doc__
 
 
 def dcgain(*args):
-    '''
-    Compute the gain of the system in steady state.
+    '''Compute the gain of the system in steady state
 
     The function takes either 1, 2, 3, or 4 parameters:
 
