@@ -34,9 +34,6 @@
 # IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-# Python 3 compatibility
-from __future__ import print_function
-
 import numpy as np
 import matplotlib.pyplot as mpl
 
@@ -73,7 +70,7 @@ def phase_plot(odefun, X=None, Y=None, scale=1, X0=None, T=None,
     func : callable(x, t, ...)
         Computes the time derivative of y (compatible with odeint).
         The function should be the same for as used for
-        scipy.integrate.  Namely, it should be a function of the form
+        :mod:`scipy.integrate`.  Namely, it should be a function of the form
         dxdt = F(x, t) that accepts a state x of dimension 2 and
         returns a derivative dx/dt of dimension 2.
 
@@ -118,9 +115,6 @@ def phase_plot(odefun, X=None, Y=None, scale=1, X0=None, T=None,
     --------
     box_grid : construct box-shaped grid of initial conditions
 
-    Examples
-    --------
-
     """
 
     #
