@@ -70,7 +70,24 @@ def h2syn(P, nmeas, ncon):
 
     Examples
     --------
-    >>> K = h2syn(P,nmeas,ncon)
+    >>> # Unstable first order SISI system
+    >>> G = ct.tf([1], [1, -1], inputs=['u'], outputs=['y'])
+    >>> max(G.poles()) < 0  # Is G stable?
+    False
+
+    >>> # Create partitioned system with trivial unity systems
+    >>> P11 = ct.tf([0], [1], inputs=['w'], outputs=['z'])
+    >>> P12 = ct.tf([1], [1], inputs=['u'], outputs=['z'])
+    >>> P21 = ct.tf([1], [1], inputs=['w'], outputs=['y'])
+    >>> P22 = G
+    >>> P = ct.interconnect([P11, P12, P21, P22],
+    ...                     inplist=['w', 'u'], outlist=['z', 'y'])
+
+    >>> # Synthesize H2 optimal stabilizing controller
+    >>> K = ct.h2syn(P, nmeas=1, ncon=1)
+    >>> T = ct.feedback(G, K, sign=1)
+    >>> max(T.poles()) < 0  # Is T stable?
+    True
 
     """
     # Check for ss system object, need a utility for this?
@@ -119,8 +136,8 @@ def hinfsyn(P, nmeas, ncon):
     rcond: 4-vector, reciprocal condition estimates of:
         1: control transformation matrix
         2: measurement transformation matrix
-        3: X-Ricatti equation
-        4: Y-Ricatti equation
+        3: X-Riccati equation
+        4: Y-Riccati equation
     TODO: document significance of rcond
 
     Raises
@@ -134,7 +151,23 @@ def hinfsyn(P, nmeas, ncon):
 
     Examples
     --------
-    >>> K, CL, gam, rcond = hinfsyn(P,nmeas,ncon)
+    >>> # Unstable first order SISI system
+    >>> G = ct.tf([1], [1,-1], inputs=['u'], outputs=['y'])
+    >>> max(G.poles()) < 0
+    False
+
+    >>> # Create partitioned system with trivial unity systems
+    >>> P11 = ct.tf([0], [1], inputs=['w'], outputs=['z'])
+    >>> P12 = ct.tf([1], [1], inputs=['u'], outputs=['z'])
+    >>> P21 = ct.tf([1], [1], inputs=['w'], outputs=['y'])
+    >>> P22 = G
+    >>> P = ct.interconnect([P11, P12, P21, P22], inplist=['w', 'u'], outlist=['z', 'y'])
+
+    >>> # Synthesize Hinf optimal stabilizing controller
+    >>> K, CL, gam, rcond = ct.hinfsyn(P, nmeas=1, ncon=1)
+    >>> T = ct.feedback(G, K, sign=1)
+    >>> max(T.poles()) < 0
+    True
 
     """
 
@@ -206,12 +239,12 @@ def _size_as_needed(w, wname, n):
     if w is not None:
         if not isinstance(w, StateSpace):
             w = ss(w)
-        if 1 == w.inputs and 1 == w.outputs:
+        if 1 == w.ninputs and 1 == w.noutputs:
             w = append(*(w,) * n)
         else:
-            if w.inputs != n:
+            if w.ninputs != n:
                 msg = ("{}: weighting function has {} inputs, expected {}".
-                       format(wname, w.inputs, n))
+                       format(wname, w.ninputs, n))
                 raise ValueError(msg)
     else:
         w = ss([], [], [], [])
@@ -221,28 +254,33 @@ def _size_as_needed(w, wname, n):
 def augw(g, w1=None, w2=None, w3=None):
     """Augment plant for mixed sensitivity problem.
 
-    Parameters
-    ----------
-    g: LTI object, ny-by-nu
-    w1: weighting on S; None, scalar, or k1-by-ny LTI object
-    w2: weighting on KS; None, scalar, or k2-by-nu LTI object
-    w3: weighting on T; None, scalar, or k3-by-ny LTI object
-    p: augmented plant; StateSpace object
-
     If a weighting is None, no augmentation is done for it.  At least
     one weighting must not be None.
 
     If a weighting w is scalar, it will be replaced by I*w, where I is
     ny-by-ny for w1 and w3, and nu-by-nu for w2.
 
+    Parameters
+    ----------
+    g: LTI object, ny-by-nu
+        Plant
+    w1: None, scalar, or k1-by-ny LTI object
+        Weighting on S
+    w2: None, scalar, or k2-by-nu LTI object
+        Weighting on KS
+    w3: None, scalar, or k3-by-ny LTI object
+        Weighting on T
+
     Returns
     -------
-    p: plant augmented with weightings, suitable for submission to hinfsyn or h2syn.
+    p: StateSpace
+        Plant augmented with weightings, suitable for submission to hinfsyn or
+        h2syn.
 
     Raises
     ------
     ValueError
-        - if all weightings are None
+        If all weightings are None
 
     See Also
     --------
@@ -253,8 +291,8 @@ def augw(g, w1=None, w2=None, w3=None):
 
     if w1 is None and w2 is None and w3 is None:
         raise ValueError("At least one weighting must not be None")
-    ny = g.outputs
-    nu = g.inputs
+    ny = g.noutputs
+    nu = g.ninputs
 
     w1, w2, w3 = [_size_as_needed(w, wname, n)
                   for w, wname, n in zip((w1, w2, w3),
@@ -278,13 +316,13 @@ def augw(g, w1=None, w2=None, w3=None):
 
     sysall = append(w1, w2, w3, Ie, g, Iu)
 
-    niw1 = w1.inputs
-    niw2 = w2.inputs
-    niw3 = w3.inputs
+    niw1 = w1.ninputs
+    niw2 = w2.ninputs
+    niw3 = w3.ninputs
 
-    now1 = w1.outputs
-    now2 = w2.outputs
-    now3 = w3.outputs
+    now1 = w1.noutputs
+    now2 = w2.noutputs
+    now3 = w3.noutputs
 
     q = np.zeros((niw1 + niw2 + niw3 + ny + nu, 2))
     q[:, 0] = np.arange(1, q.shape[0] + 1)
@@ -331,25 +369,35 @@ def mixsyn(g, w1=None, w2=None, w3=None):
 
     Parameters
     ----------
-    g: LTI; the plant for which controller must be synthesized
-    w1: weighting on s = (1+g*k)**-1; None, or scalar or k1-by-ny LTI
-    w2: weighting on k*s; None, or scalar or k2-by-nu LTI
-    w3: weighting on t = g*k*(1+g*k)**-1; None, or scalar or k3-by-ny LTI
-    At least one of w1, w2, and w3 must not be None.
+    g: LTI
+        The plant for which controller must be synthesized
+    w1: None, or scalar or k1-by-ny LTI
+        Weighting on S = (1+G*K)**-1
+    w2: None, or scalar or k2-by-nu LTI
+        Weighting on K*S
+    w3: None, or scalar or k3-by-ny LTI
+        Weighting on T = G*K*(1+G*K)**-1;
 
     Returns
     -------
-    k: synthesized controller; StateSpace object
-    cl: closed system mapping evaluation inputs to evaluation outputs; if 
-    p is the augmented plant, with
-        [z] = [p11 p12] [w], 
-        [y]   [p21   g] [u]
-    then cl is the system from w->z with u=-k*y.  StateSpace object.
+    k: StateSpace
+        Synthesized controller;
+    cl: StateSpace
+        Closed system mapping evaluation inputs to evaluation outputs.
 
-    info: tuple with entries, in order,
-        - gamma: scalar; H-infinity norm of cl
-        - rcond: array; estimates of reciprocal condition numbers
-          computed during synthesis.  See hinfsyn for details
+        Let p be the augmented plant, with::
+
+            [z] = [p11 p12] [w]
+            [y]   [p21   g] [u]
+
+        then cl is the system from w->z with `u = -k*y`.
+
+    info: tuple
+        gamma: scalar
+            H-infinity norm of cl
+        rcond: array
+            Estimates of reciprocal condition numbers
+            computed during synthesis.  See hinfsyn for details
 
     If a weighting w is scalar, it will be replaced by I*w, where I is
     ny-by-ny for w1 and w3, and nu-by-nu for w2.
@@ -358,8 +406,8 @@ def mixsyn(g, w1=None, w2=None, w3=None):
     --------
     hinfsyn, augw
     """
-    nmeas = g.outputs
-    ncon = g.inputs
+    nmeas = g.noutputs
+    ncon = g.ninputs
     p = augw(g, w1, w2, w3)
 
     k, cl, gamma, rcond = hinfsyn(p, nmeas, ncon)
