@@ -41,13 +41,19 @@
 
 # External packages and modules
 import numpy as np
+import scipy as sp
+import warnings
 
 from . import statesp
 from .mateqn import care, dare, _check_shape
 from .statesp import StateSpace, _ssmatrix, _convert_to_statespace
-from .lti import LTI, isdtime, isctime
+from .lti import LTI
+from .namedio import isdtime, isctime, _process_indices, _process_labels
+from .iosys import InputOutputSystem, NonlinearIOSystem, LinearIOSystem, \
+    interconnect, ss
 from .exception import ControlSlycot, ControlArgument, ControlDimension, \
     ControlNotImplemented
+from .config import _process_legacy_keyword
 
 # Make sure we have access to the right slycot routines
 try:
@@ -68,8 +74,8 @@ except ImportError:
     sb03od = None
 
 
-__all__ = ['ctrb', 'obsv', 'gram', 'place', 'place_varga', 'lqr', 'lqe',
-           'dlqr', 'dlqe', 'acker']
+__all__ = ['ctrb', 'obsv', 'gram', 'place', 'place_varga', 'lqr',
+           'dlqr', 'acker', 'create_statefbk_iosystem']
 
 
 # Pole placement
@@ -117,16 +123,12 @@ def place(A, B, p):
     --------
     >>> A = [[-1, -1], [0, 1]]
     >>> B = [[0], [1]]
-    >>> K = place(A, B, [-2, -5])
+    >>> K = ct.place(A, B, [-2, -5])
 
     See Also
     --------
     place_varga, acker
 
-    Notes
-    -----
-    The return type for 2D arrays depends on the default class set for
-    state space operations.  See :func:`~control.use_numpy_matrix`.
     """
     from scipy.signal import place_poles
 
@@ -258,276 +260,6 @@ def place_varga(A, B, p, dtime=False, alpha=None):
     return _ssmatrix(-F)
 
 
-# contributed by Sawyer B. Fuller <minster@uw.edu>
-def lqe(*args, **keywords):
-    """lqe(A, G, C, QN, RN, [, NN])
-
-    Linear quadratic estimator design (Kalman filter) for continuous-time
-    systems. Given the system
-
-    .. math::
-
-        x &= Ax + Bu + Gw \\\\
-        y &= Cx + Du + v
-
-    with unbiased process noise w and measurement noise v with covariances
-
-    .. math::       E{ww'} = QN,    E{vv'} = RN,    E{wv'} = NN
-
-    The lqe() function computes the observer gain matrix L such that the
-    stationary (non-time-varying) Kalman filter
-
-    .. math:: x_e = A x_e + B u + L(y - C x_e - D u)
-
-    produces a state estimate x_e that minimizes the expected squared error
-    using the sensor measurements y. The noise cross-correlation `NN` is
-    set to zero when omitted.
-
-    The function can be called with either 3, 4, 5, or 6 arguments:
-
-    * ``L, P, E = lqe(sys, QN, RN)``
-    * ``L, P, E = lqe(sys, QN, RN, NN)``
-    * ``L, P, E = lqe(A, G, C, QN, RN)``
-    * ``L, P, E = lqe(A, G, C, QN, RN, NN)``
-
-    where `sys` is an `LTI` object, and `A`, `G`, `C`, `QN`, `RN`, and `NN`
-    are 2D arrays or matrices of appropriate dimension.
-
-    Parameters
-    ----------
-    A, G, C : 2D array_like
-        Dynamics, process noise (disturbance), and output matrices
-    sys : LTI (StateSpace or TransferFunction)
-        Linear I/O system, with the process noise input taken as the system
-        input.
-    QN, RN : 2D array_like
-        Process and sensor noise covariance matrices
-    NN : 2D array, optional
-        Cross covariance matrix.  Not currently implemented.
-    method : str, optional
-        Set the method used for computing the result.  Current methods are
-        'slycot' and 'scipy'.  If set to None (default), try 'slycot' first
-        and then 'scipy'.
-
-    Returns
-    -------
-    L : 2D array (or matrix)
-        Kalman estimator gain
-    P : 2D array (or matrix)
-        Solution to Riccati equation
-
-        .. math::
-
-            A P + P A^T - (P C^T + G N) R^{-1}  (C P + N^T G^T) + G Q G^T = 0
-
-    E : 1D array
-        Eigenvalues of estimator poles eig(A - L C)
-
-    Notes
-    -----
-    1. If the first argument is an LTI object, then this object will be used
-       to define the dynamics, noise and output matrices.  Furthermore, if
-       the LTI object corresponds to a discrete time system, the ``dlqe()``
-       function will be called.
-
-    2. The return type for 2D arrays depends on the default class set for
-       state space operations.  See :func:`~control.use_numpy_matrix`.
-
-    Examples
-    --------
-    >>> L, P, E = lqe(A, G, C, QN, RN)
-    >>> L, P, E = lqe(A, G, C, Q, RN, NN)
-
-    See Also
-    --------
-    lqr, dlqe, dlqr
-
-    """
-
-    # TODO: incorporate cross-covariance NN, something like this,
-    # which doesn't work for some reason
-    # if NN is None:
-    #    NN = np.zeros(QN.size(0),RN.size(1))
-    # NG = G @ NN
-
-    #
-    # Process the arguments and figure out what inputs we received
-    #
-
-    # Get the method to use (if specified as a keyword)
-    method = keywords.get('method', None)
-
-    # Get the system description
-    if (len(args) < 3):
-        raise ControlArgument("not enough input arguments")
-
-    # If we were passed a discrete time system as the first arg, use dlqe()
-    if isinstance(args[0], LTI) and isdtime(args[0], strict=True):
-        # Call dlqe
-        return dlqe(*args, **keywords)
-
-    # If we were passed a state space  system, use that to get system matrices
-    if isinstance(args[0], StateSpace):
-        A = np.array(args[0].A, ndmin=2, dtype=float)
-        G = np.array(args[0].B, ndmin=2, dtype=float)
-        C = np.array(args[0].C, ndmin=2, dtype=float)
-        index = 1
-
-    elif isinstance(args[0], LTI):
-        # Don't allow other types of LTI systems
-        raise ControlArgument("LTI system must be in state space form")
-
-    else:
-        # Arguments should be A and B matrices
-        A = np.array(args[0], ndmin=2, dtype=float)
-        G = np.array(args[1], ndmin=2, dtype=float)
-        C = np.array(args[2], ndmin=2, dtype=float)
-        index = 3
-
-    # Get the weighting matrices (converting to matrices, if needed)
-    QN = np.array(args[index], ndmin=2, dtype=float)
-    RN = np.array(args[index+1], ndmin=2, dtype=float)
-
-    # Get the cross-covariance matrix, if given
-    if (len(args) > index + 2):
-        NN = np.array(args[index+2], ndmin=2, dtype=float)
-        raise ControlNotImplemented("cross-covariance not implemented")
-
-    else:
-        # For future use (not currently used below)
-        NN = np.zeros((QN.shape[0], RN.shape[1]))
-
-    # Check dimensions of G (needed before calling care())
-    _check_shape("QN", QN, G.shape[1], G.shape[1])
-
-    # Compute the result (dimension and symmetry checking done in care())
-    P, E, LT = care(A.T, C.T, G @ QN @ G.T, RN, method=method,
-                    B_s="C", Q_s="QN", R_s="RN", S_s="NN")
-    return _ssmatrix(LT.T), _ssmatrix(P), E
-
-
-# contributed by Sawyer B. Fuller <minster@uw.edu>
-def dlqe(*args, **keywords):
-    """dlqe(A, G, C, QN, RN, [, N])
-
-    Linear quadratic estimator design (Kalman filter) for discrete-time
-    systems. Given the system
-
-    .. math::
-
-        x[n+1] &= Ax[n] + Bu[n] + Gw[n] \\\\
-        y[n] &= Cx[n] + Du[n] + v[n]
-
-    with unbiased process noise w and measurement noise v with covariances
-
-    .. math::       E{ww'} = QN,    E{vv'} = RN,    E{wv'} = NN
-
-    The dlqe() function computes the observer gain matrix L such that the
-    stationary (non-time-varying) Kalman filter
-
-    .. math:: x_e[n+1] = A x_e[n] + B u[n] + L(y[n] - C x_e[n] - D u[n])
-
-    produces a state estimate x_e[n] that minimizes the expected squared error
-    using the sensor measurements y. The noise cross-correlation `NN` is
-    set to zero when omitted.
-
-    Parameters
-    ----------
-    A, G : 2D array_like
-        Dynamics and noise input matrices
-    QN, RN : 2D array_like
-        Process and sensor noise covariance matrices
-    NN : 2D array, optional
-        Cross covariance matrix (not yet supported)
-    method : str, optional
-        Set the method used for computing the result.  Current methods are
-        'slycot' and 'scipy'.  If set to None (default), try 'slycot' first
-        and then 'scipy'.
-
-    Returns
-    -------
-    L : 2D array (or matrix)
-        Kalman estimator gain
-    P : 2D array (or matrix)
-        Solution to Riccati equation
-
-        .. math::
-
-            A P + P A^T - (P C^T + G N) R^{-1}  (C P + N^T G^T) + G Q G^T = 0
-
-    E : 1D array
-        Eigenvalues of estimator poles eig(A - L C)
-
-    Notes
-    -----
-    The return type for 2D arrays depends on the default class set for
-    state space operations.  See :func:`~control.use_numpy_matrix`.
-
-    Examples
-    --------
-    >>> L, P, E = dlqe(A, G, C, QN, RN)
-    >>> L, P, E = dlqe(A, G, C, QN, RN, NN)
-
-    See Also
-    --------
-    dlqr, lqe, lqr
-
-    """
-
-    #
-    # Process the arguments and figure out what inputs we received
-    #
-
-    # Get the method to use (if specified as a keyword)
-    method = keywords.get('method', None)
-
-    # Get the system description
-    if (len(args) < 3):
-        raise ControlArgument("not enough input arguments")
-
-    # If we were passed a continus time system as the first arg, raise error
-    if isinstance(args[0], LTI) and isctime(args[0], strict=True):
-        raise ControlArgument("dlqr() called with a continuous time system")
-
-    # If we were passed a state space  system, use that to get system matrices
-    if isinstance(args[0], StateSpace):
-        A = np.array(args[0].A, ndmin=2, dtype=float)
-        G = np.array(args[0].B, ndmin=2, dtype=float)
-        C = np.array(args[0].C, ndmin=2, dtype=float)
-        index = 1
-
-    elif isinstance(args[0], LTI):
-        # Don't allow other types of LTI systems
-        raise ControlArgument("LTI system must be in state space form")
-
-    else:
-        # Arguments should be A and B matrices
-        A = np.array(args[0], ndmin=2, dtype=float)
-        G = np.array(args[1], ndmin=2, dtype=float)
-        C = np.array(args[2], ndmin=2, dtype=float)
-        index = 3
-
-    # Get the weighting matrices (converting to matrices, if needed)
-    QN = np.array(args[index], ndmin=2, dtype=float)
-    RN = np.array(args[index+1], ndmin=2, dtype=float)
-
-    # TODO: incorporate cross-covariance NN, something like this,
-    # which doesn't work for some reason
-    # if NN is None:
-    #    NN = np.zeros(QN.size(0),RN.size(1))
-    # NG = G @ NN
-    if len(args) > index + 2:
-        NN = np.array(args[index+2], ndmin=2, dtype=float)
-        raise ControlNotImplemented("cross-covariance not yet implememented")
-
-    # Check dimensions of G (needed before calling care())
-    _check_shape("QN", QN, G.shape[1], G.shape[1])
-
-    # Compute the result (dimension and symmetry checking done in dare())
-    P, E, LT = dare(A.T, C.T, G @ QN @ G.T, RN, method=method,
-                    B_s="C", Q_s="QN", R_s="RN", S_s="NN")
-    return _ssmatrix(LT.T), _ssmatrix(P), E
-
 # Contributed by Roberto Bucher <roberto.bucher@supsi.ch>
 def acker(A, B, poles):
     """Pole placement using Ackermann method
@@ -576,7 +308,7 @@ def acker(A, B, poles):
     return _ssmatrix(K)
 
 
-def lqr(*args, **keywords):
+def lqr(*args, **kwargs):
     """lqr(A, B, Q, R[, N])
 
     Linear quadratic regulator design
@@ -606,6 +338,14 @@ def lqr(*args, **keywords):
         State and input weight matrices
     N : 2D array, optional
         Cross weight matrix
+    integral_action : ndarray, optional
+        If this keyword is specified, the controller includes integral
+        action in addition to state feedback.  The value of the
+        `integral_action` keyword should be an ndarray that will be
+        multiplied by the current state to generate the error for the
+        internal integrator states of the control law.  The number of
+        outputs that are to be integrated must match the number of
+        additional rows and columns in the `Q` matrix.
     method : str, optional
         Set the method used for computing the result.  Current methods are
         'slycot' and 'scipy'.  If set to None (default), try 'slycot' first
@@ -636,26 +376,23 @@ def lqr(*args, **keywords):
 
     Examples
     --------
-    >>> K, S, E = lqr(sys, Q, R, [N])
-    >>> K, S, E = lqr(A, B, Q, R, [N])
+    >>> K, S, E = lqr(sys, Q, R, [N])                           # doctest: +SKIP
+    >>> K, S, E = lqr(A, B, Q, R, [N])                          # doctest: +SKIP
 
     """
     #
     # Process the arguments and figure out what inputs we received
     #
 
-    # Get the method to use (if specified as a keyword)
-    method = keywords.get('method', None)
+    # If we were passed a discrete time system as the first arg, use dlqr()
+    if isinstance(args[0], LTI) and isdtime(args[0], strict=True):
+        # Call dlqr
+        return dlqr(*args, **kwargs)
 
     # Get the system description
     if (len(args) < 3):
         raise ControlArgument("not enough input arguments")
 
-    # If we were passed a discrete time system as the first arg, use dlqr()
-    if isinstance(args[0], LTI) and isdtime(args[0], strict=True):
-        # Call dlqr
-        return dlqr(*args, **keywords)
-
     # If we were passed a state space  system, use that to get system matrices
     if isinstance(args[0], StateSpace):
         A = np.array(args[0].A, ndmin=2, dtype=float)
@@ -680,12 +417,47 @@ def lqr(*args, **keywords):
     else:
         N = None
 
+    #
+    # Process keywords
+    #
+
+    # Get the method to use (if specified as a keyword)
+    method = kwargs.pop('method', None)
+
+    # See if we should augment the controller with integral feedback
+    integral_action = kwargs.pop('integral_action', None)
+    if integral_action is not None:
+        # Figure out the size of the system
+        nstates = A.shape[0]
+        ninputs = B.shape[1]
+
+        # Make sure that the integral action argument is the right type
+        if not isinstance(integral_action, np.ndarray):
+            raise ControlArgument("Integral action must pass an array")
+        elif integral_action.shape[1] != nstates:
+            raise ControlArgument(
+                "Integral gain size must match system state size")
+
+        # Process the states to be integrated
+        nintegrators = integral_action.shape[0]
+        C = integral_action
+
+        # Augment the system with integrators
+        A = np.block([
+            [A, np.zeros((nstates, nintegrators))],
+            [C, np.zeros((nintegrators, nintegrators))]
+        ])
+        B = np.vstack([B, np.zeros((nintegrators, ninputs))])
+
+    if kwargs:
+        raise TypeError("unrecognized keywords: ", str(kwargs))
+
     # Compute the result (dimension and symmetry checking done in care())
     X, L, G = care(A, B, Q, R, N, None, method=method, S_s="N")
     return G, X, L
 
 
-def dlqr(*args, **keywords):
+def dlqr(*args, **kwargs):
     """dlqr(A, B, Q, R[, N])
 
     Discrete-time linear quadratic regulator design
@@ -693,7 +465,7 @@ def dlqr(*args, **keywords):
     The dlqr() function computes the optimal state feedback controller
     u[n] = - K x[n] that minimizes the quadratic cost
 
-    .. math:: J = \\Sum_0^\\infty (x[n]' Q x[n] + u[n]' R u[n] + 2 x[n]' N u[n])
+    .. math:: J = \\sum_0^\\infty (x[n]' Q x[n] + u[n]' R u[n] + 2 x[n]' N u[n])
 
     The function can be called with either 3, 4, or 5 arguments:
 
@@ -716,6 +488,18 @@ def dlqr(*args, **keywords):
         State and input weight matrices
     N : 2D array, optional
         Cross weight matrix
+    integral_action : ndarray, optional
+        If this keyword is specified, the controller includes integral
+        action in addition to state feedback.  The value of the
+        `integral_action` keyword should be an ndarray that will be
+        multiplied by the current state to generate the error for the
+        internal integrator states of the control law.  The number of
+        outputs that are to be integrated must match the number of
+        additional rows and columns in the `Q` matrix.
+    method : str, optional
+        Set the method used for computing the result.  Current methods are
+        'slycot' and 'scipy'.  If set to None (default), try 'slycot' first
+        and then 'scipy'.
 
     Returns
     -------
@@ -737,17 +521,14 @@ def dlqr(*args, **keywords):
 
     Examples
     --------
-    >>> K, S, E = dlqr(dsys, Q, R, [N])
-    >>> K, S, E = dlqr(A, B, Q, R, [N])
+    >>> K, S, E = dlqr(dsys, Q, R, [N])                         # doctest: +SKIP
+    >>> K, S, E = dlqr(A, B, Q, R, [N])                         # doctest: +SKIP
     """
 
     #
     # Process the arguments and figure out what inputs we received
     #
 
-    # Get the method to use (if specified as a keyword)
-    method = keywords.get('method', None)
-
     # Get the system description
     if (len(args) < 3):
         raise ControlArgument("not enough input arguments")
@@ -780,11 +561,403 @@ def dlqr(*args, **keywords):
     else:
         N = np.zeros((Q.shape[0], R.shape[1]))
 
+    #
+    # Process keywords
+    #
+
+    # Get the method to use (if specified as a keyword)
+    method = kwargs.pop('method', None)
+
+    # See if we should augment the controller with integral feedback
+    integral_action = kwargs.pop('integral_action', None)
+    if integral_action is not None:
+        # Figure out the size of the system
+        nstates = A.shape[0]
+        ninputs = B.shape[1]
+
+        if not isinstance(integral_action, np.ndarray):
+            raise ControlArgument("Integral action must pass an array")
+        elif integral_action.shape[1] != nstates:
+            raise ControlArgument(
+                "Integral gain size must match system state size")
+        else:
+            nintegrators = integral_action.shape[0]
+            C = integral_action
+
+            # Augment the system with integrators
+            A = np.block([
+                [A, np.zeros((nstates, nintegrators))],
+                [C, np.eye(nintegrators)]
+            ])
+            B = np.vstack([B, np.zeros((nintegrators, ninputs))])
+
+    if kwargs:
+        raise TypeError("unrecognized keywords: ", str(kwargs))
+
     # Compute the result (dimension and symmetry checking done in dare())
     S, E, K = dare(A, B, Q, R, N, method=method, S_s="N")
     return _ssmatrix(K), _ssmatrix(S), E
 
 
+# Function to create an I/O sytems representing a state feedback controller
+def create_statefbk_iosystem(
+        sys, gain, integral_action=None, estimator=None, controller_type=None,
+        xd_labels=None, ud_labels=None, gainsched_indices=None,
+        gainsched_method='linear', control_indices=None, state_indices=None,
+        name=None, inputs=None, outputs=None, states=None, **kwargs):
+    """Create an I/O system using a (full) state feedback controller
+
+    This function creates an input/output system that implements a
+    state feedback controller of the form
+
+        u = ud - K_p (x - xd) - K_i integral(C x - C x_d)
+
+    It can be called in the form
+
+        ctrl, clsys = ct.create_statefbk_iosystem(sys, K)
+
+    where `sys` is the process dynamics and `K` is the state (+ integral)
+    feedback gain (eg, from LQR).  The function returns the controller
+    `ctrl` and the closed loop systems `clsys`, both as I/O systems.
+
+    A gain scheduled controller can also be created, by passing a list of
+    gains and a corresponding list of values of a set of scheduling
+    variables.  In this case, the controller has the form
+
+        u = ud - K_p(mu) (x - xd) - K_i(mu) integral(C x - C x_d)
+
+    where mu represents the scheduling variable.
+
+    Parameters
+    ----------
+    sys : InputOutputSystem
+        The I/O system that represents the process dynamics.  If no estimator
+        is given, the output of this system should represent the full state.
+
+    gain : ndarray or tuple
+        If an array is given, it represents the state feedback gain (K).
+        This matrix defines the gains to be applied to the system.  If
+        `integral_action` is None, then the dimensions of this array
+        should be (sys.ninputs, sys.nstates).  If `integral action` is
+        set to a matrix or a function, then additional columns
+        represent the gains of the integral states of the controller.
+
+        If a tuple is given, then it specifies a gain schedule.  The tuple
+        should be of the form `(gains, points)` where gains is a list of
+        gains :math:`K_j` and points is a list of values :math:`\\mu_j` at
+        which the gains are computed.  The `gainsched_indices` parameter
+        should be used to specify the scheduling variables.
+
+    xd_labels, ud_labels : str or list of str, optional
+        Set the name of the signals to use for the desired state and
+        inputs.  If a single string is specified, it should be a
+        format string using the variable `i` as an index.  Otherwise,
+        a list of strings matching the size of xd and ud,
+        respectively, should be used.  Default is "xd[{i}]" for
+        xd_labels and "ud[{i}]" for ud_labels.  These settings can
+        also be overriden using the `inputs` keyword.
+
+    integral_action : ndarray, optional
+        If this keyword is specified, the controller can include integral
+        action in addition to state feedback.  The value of the
+        `integral_action` keyword should be an ndarray that will be
+        multiplied by the current and desired state to generate the error
+        for the internal integrator states of the control law.
+
+    estimator : InputOutputSystem, optional
+        If an estimator is provided, use the states of the estimator as
+        the system inputs for the controller.
+
+    gainsched_indices : int, slice, or list of int or str, optional
+        If a gain scheduled controller is specified, specify the indices of
+        the controller input to use for scheduling the gain. The input to
+        the controller is the desired state xd, the desired input ud, and
+        the system state x (or state estimate xhat, if an estimator is
+        given). If value is an integer `q`, the first `q` values of the
+        [xd, ud, x] vector are used.  Otherwise, the value should be a
+        slice or a list of indices.  The list of indices can be specified
+        as either integer offsets or as signal names.  The default is to
+        use the desired state xd.
+
+    gainsched_method : str, optional
+        The method to use for gain scheduling.  Possible values are 'linear'
+        (default), 'nearest', and 'cubic'.  More information is available in
+        :func:`scipy.interpolate.griddata`. For points outside of the convex
+        hull of the scheduling points, the gain at the nearest point is
+        used.
+
+    controller_type : 'linear' or 'nonlinear', optional
+        Set the type of controller to create. The default for a linear gain
+        is a linear controller implementing the LQR regulator. If the type
+        is 'nonlinear', a :class:NonlinearIOSystem is created instead, with
+        the gain `K` as a parameter (allowing modifications of the gain at
+        runtime). If the gain parameter is a tuple, then a nonlinear,
+        gain-scheduled controller is created.
+
+    Returns
+    -------
+    ctrl : InputOutputSystem
+        Input/output system representing the controller.  This system
+        takes as inputs the desired state `xd`, the desired input
+        `ud`, and either the system state `x` or the estimated state
+        `xhat`.  It outputs the controller action `u` according to the
+        formula :math:`u = u_d - K(x - x_d)`.  If the keyword
+        `integral_action` is specified, then an additional set of
+        integrators is included in the control system (with the gain
+        matrix `K` having the integral gains appended after the state
+        gains).  If a gain scheduled controller is specified, the gain
+        (proportional and integral) are evaluated using the scheduling
+        variables specified by `gainsched_indices`.
+
+    clsys : InputOutputSystem
+        Input/output system representing the closed loop system.  This
+        systems takes as inputs the desired trajectory `(xd, ud)` and
+        outputs the system state `x` and the applied input `u`
+        (vertically stacked).
+
+    Other Parameters
+    ----------------
+    control_indices : int, slice, or list of int or str, optional
+        Specify the indices of the system inputs that should be determined
+        by the state feedback controller.  If value is an integer `m`, the
+        first `m` system inputs are used.  Otherwise, the value should be a
+        slice or a list of indices.  The list of indices can be specified
+        as either integer offsets or as system input signal names.  If not
+        specified, defaults to the system inputs.
+
+    state_indices : int, slice, or list of int or str, optional
+        Specify the indices of the system (or estimator) outputs that should
+        be used by the state feedback controller.  If value is an integer
+        `n`, the first `n` system states are used.  Otherwise, the value
+        should be a slice or a list of indices.  The list of indices can be
+        specified as either integer offsets or as estimator/system output
+        signal names.  If not specified, defaults to the system states.
+
+    inputs, outputs : str, or list of str, optional
+        List of strings that name the individual signals of the transformed
+        system.  If not given, the inputs and outputs are the same as the
+        original system.
+
+    name : string, optional
+        System name. If unspecified, a generic name <sys[id]> is generated
+        with a unique integer id.
+
+    """
+    # Make sure that we were passed an I/O system as an input
+    if not isinstance(sys, InputOutputSystem):
+        raise ControlArgument("Input system must be I/O system")
+
+    # Process (legacy) keywords
+    controller_type = _process_legacy_keyword(
+        kwargs, 'type', 'controller_type', controller_type)
+    if kwargs:
+        raise TypeError("unrecognized keywords: ", str(kwargs))
+
+    # Figure out what inputs to the system to use
+    control_indices = _process_indices(
+        control_indices, 'control', sys.input_labels, sys.ninputs)
+    sys_ninputs = len(control_indices)
+
+    # Decide what system is going to pass the states to the controller
+    if estimator is None:
+        estimator = sys
+
+    # Figure out what outputs (states) from the system/estimator to use
+    state_indices = _process_indices(
+        state_indices, 'state', estimator.state_labels, sys.nstates)
+    sys_nstates = len(state_indices)
+
+    # Make sure the system/estimator states are proper dimension
+    if estimator.noutputs < sys_nstates:
+        # If no estimator, make sure that the system has all states as outputs
+        raise ControlArgument(
+            ("system" if estimator == sys else "estimator") +
+            " output must include the full state")
+    elif estimator == sys:
+        # Issue a warning if we can't verify state output
+        if (isinstance(sys, NonlinearIOSystem) and sys.outfcn is not None) or \
+           (isinstance(sys, StateSpace) and
+            not (np.all(sys.C[np.ix_(state_indices, state_indices)] ==
+                        np.eye(sys_nstates)) and
+                 np.all(sys.D[state_indices, :] == 0))):
+            warnings.warn("cannot verify system output is system state")
+
+    # See whether we should implement integral action
+    nintegrators = 0
+    if integral_action is not None:
+        if not isinstance(integral_action, np.ndarray):
+            raise ControlArgument("Integral action must pass an array")
+        elif integral_action.shape[1] != sys_nstates:
+            raise ControlArgument(
+                "Integral gain size must match system state size")
+        else:
+            nintegrators = integral_action.shape[0]
+            C = integral_action
+    else:
+        # Create a C matrix with no outputs, just in case update gets called
+        C = np.zeros((0, sys_nstates))
+
+    # Check to make sure that state feedback has the right shape
+    if isinstance(gain, np.ndarray):
+        K = gain
+        if K.shape != (sys_ninputs, estimator.noutputs + nintegrators):
+            raise ControlArgument(
+                f'control gain must be an array of size {sys_ninputs}'
+                f' x {sys_nstates}' +
+                (f'+{nintegrators}' if nintegrators > 0 else ''))
+        gainsched = False
+
+    elif isinstance(gain, tuple):
+        # Check for gain scheduled controller
+        if len(gain) != 2:
+            raise ControlArgument("gain must be a 2-tuple for gain scheduling")
+        gains, points = gain[0:2]
+
+        # Stack gains and points if past as a list
+        gains = np.stack(gains)
+        points = np.stack(points)
+        gainsched=True
+
+    else:
+        raise ControlArgument("gain must be an array or a tuple")
+
+    # Decide on the type of system to create
+    if gainsched and controller_type == 'linear':
+        raise ControlArgument(
+            "controller_type 'linear' not allowed for"
+            " gain scheduled controller")
+    elif controller_type is None:
+        controller_type = 'nonlinear' if gainsched else 'linear'
+    elif controller_type not in {'linear', 'nonlinear'}:
+        raise ControlArgument(f"unknown controller_type '{controller_type}'")
+
+    # Figure out the labels to use
+    xd_labels = _process_labels(
+        xd_labels, 'xd', ['xd[{i}]'.format(i=i) for i in range(sys_nstates)])
+    ud_labels = _process_labels(
+        ud_labels, 'ud', ['ud[{i}]'.format(i=i) for i in range(sys_ninputs)])
+
+    # Create the signal and system names
+    if inputs is None:
+        inputs = xd_labels + ud_labels + estimator.output_labels
+    if outputs is None:
+        outputs = [sys.input_labels[i] for i in control_indices]
+    if states is None:
+        states = nintegrators
+
+    # Process gainscheduling variables, if present
+    if gainsched:
+        # Create a copy of the scheduling variable indices (default = xd)
+        gainsched_indices = _process_indices(
+            gainsched_indices, 'gainsched', inputs, sys_nstates)
+
+        # If points is a 1D list, convert to 2D
+        if points.ndim == 1:
+            points = points.reshape(-1, 1)
+
+        # Make sure the scheduling variable indices are the right length
+        if len(gainsched_indices) != points.shape[1]:
+            raise ControlArgument(
+                "length of gainsched_indices must match dimension of"
+                " scheduling variables")
+
+        # Create interpolating function
+        if points.shape[1] < 2:
+            _interp = sp.interpolate.interp1d(
+                points[:, 0], gains, axis=0, kind=gainsched_method)
+            _nearest = sp.interpolate.interp1d(
+                points[:, 0], gains, axis=0, kind='nearest')
+        elif gainsched_method == 'nearest':
+            _interp = sp.interpolate.NearestNDInterpolator(points, gains)
+            def _nearest(mu):
+                raise SystemError(f"could not find nearest gain at mu = {mu}")
+        elif gainsched_method == 'linear':
+            _interp = sp.interpolate.LinearNDInterpolator(points, gains)
+            _nearest = sp.interpolate.NearestNDInterpolator(points, gains)
+        elif gainsched_method == 'cubic':
+            _interp = sp.interpolate.CloughTocher2DInterpolator(points, gains)
+            _nearest = sp.interpolate.NearestNDInterpolator(points, gains)
+        else:
+            raise ControlArgument(
+                f"unknown gain scheduling method '{gainsched_method}'")
+
+        def _compute_gain(mu):
+            K = _interp(mu)
+            if np.isnan(K).any():
+                K = _nearest(mu)
+            return K
+
+    # Define the controller system
+    if controller_type == 'nonlinear':
+        # Create an I/O system for the state feedback gains
+        def _control_update(t, states, inputs, params):
+            # Split input into desired state, nominal input, and current state
+            xd_vec = inputs[0:sys_nstates]
+            x_vec = inputs[-sys_nstates:]
+
+            # Compute the integral error in the xy coordinates
+            return C @ (x_vec - xd_vec)
+
+        def _control_output(t, states, inputs, params):
+            if gainsched:
+                mu = inputs[gainsched_indices]
+                K_ = _compute_gain(mu)
+            else:
+                K_ = params.get('K')
+
+            # Split input into desired state, nominal input, and current state
+            xd_vec = inputs[0:sys_nstates]
+            ud_vec = inputs[sys_nstates:sys_nstates + sys_ninputs]
+            x_vec = inputs[-sys_nstates:]
+
+            # Compute the control law
+            u = ud_vec - K_[:, 0:sys_nstates] @ (x_vec - xd_vec)
+            if nintegrators > 0:
+                u -= K_[:, sys_nstates:] @ states
+
+            return u
+
+        params = {} if gainsched else {'K': K}
+        ctrl = NonlinearIOSystem(
+            _control_update, _control_output, name=name, inputs=inputs,
+            outputs=outputs, states=states, params=params)
+
+    elif controller_type == 'linear' or controller_type is None:
+        # Create the matrices implementing the controller
+        if isctime(sys):
+            # Continuous time: integrator
+            A_lqr = np.zeros((C.shape[0], C.shape[0]))
+        else:
+            # Discrete time: summer
+            A_lqr = np.eye(C.shape[0])
+        B_lqr = np.hstack([-C, np.zeros((C.shape[0], sys_ninputs)), C])
+        C_lqr = -K[:, sys_nstates:]
+        D_lqr = np.hstack([
+            K[:, 0:sys_nstates], np.eye(sys_ninputs), -K[:, 0:sys_nstates]
+        ])
+
+        ctrl = ss(
+            A_lqr, B_lqr, C_lqr, D_lqr, dt=sys.dt, name=name,
+            inputs=inputs, outputs=outputs, states=states)
+
+    else:
+        raise ControlArgument(f"unknown controller_type '{controller_type}'")
+
+    # Define the closed loop system
+    inplist=inputs[:-sys.nstates]
+    input_labels=inputs[:-sys.nstates]
+    outlist=sys.output_labels + [sys.input_labels[i] for i in control_indices]
+    output_labels=sys.output_labels + \
+        [sys.input_labels[i] for i in control_indices]
+    closed = interconnect(
+        [sys, ctrl] if estimator == sys else [sys, ctrl, estimator],
+        name=sys.name + "_" + ctrl.name, add_unused=True,
+        inplist=inplist, inputs=input_labels,
+        outlist=outlist, outputs=output_labels
+    )
+    return ctrl, closed
+
+
 def ctrb(A, B):
     """Controllabilty matrix
 
@@ -805,7 +978,10 @@ def ctrb(A, B):
 
     Examples
     --------
-    >>> C = ctrb(A, B)
+    >>> G = ct.tf2ss([1], [1, 2, 3])
+    >>> C = ct.ctrb(G.A, G.B)
+    >>> np.linalg.matrix_rank(C)
+    2
 
     """
 
@@ -841,7 +1017,11 @@ def obsv(A, C):
 
     Examples
     --------
-    >>> O = obsv(A, C)
+    >>> G = ct.tf2ss([1], [1, 2, 3])
+    >>> C = ct.obsv(G.A, G.C)
+    >>> np.linalg.matrix_rank(C)
+    2
+
     """
 
     # Convert input parameters to matrices (if they aren't already)
@@ -890,10 +1070,11 @@ def gram(sys, type):
 
     Examples
     --------
-    >>> Wc = gram(sys, 'c')
-    >>> Wo = gram(sys, 'o')
-    >>> Rc = gram(sys, 'cf'), where Wc = Rc' * Rc
-    >>> Ro = gram(sys, 'of'), where Wo = Ro' * Ro
+    >>> G = ct.rss(4)
+    >>> Wc = ct.gram(G, 'c')
+    >>> Wo = ct.gram(G, 'o')
+    >>> Rc = ct.gram(G, 'cf')  # where Wc = Rc' * Rc
+    >>> Ro = ct.gram(G, 'of')  # where Wo = Ro' * Ro
 
     """
 
