@@ -55,15 +55,25 @@ from numpy.random import rand, randn
 from numpy.linalg import solve, eigvals, matrix_rank
 from numpy.linalg.linalg import LinAlgError
 import scipy as sp
+import scipy.linalg
 from scipy.signal import cont2discrete
 from scipy.signal import StateSpace as signalStateSpace
 from warnings import warn
-from .lti import LTI, common_timebase, isdtime, _process_frequency_response
+
+from .exception import ControlSlycot
+from .frdata import FrequencyResponseData
+from .lti import LTI, _process_frequency_response
+from .namedio import common_timebase, isdtime, _process_namedio_keywords, \
+    _process_dt_keyword, NamedIOSystem
 from . import config
 from copy import deepcopy
 
-__all__ = ['StateSpace', 'ss', 'rss', 'drss', 'tf2ss', 'ssdata']
+try:
+    from slycot import ab13dd
+except ImportError:
+    ab13dd = None
 
+__all__ = ['StateSpace', 'tf2ss', 'ssdata', 'linfnorm']
 
 # Define module default parameter values
 _statesp_defaults = {
@@ -154,17 +164,19 @@ def _f2s(f):
 
 
 class StateSpace(LTI):
-    """StateSpace(A, B, C, D[, dt])
+    r"""StateSpace(A, B, C, D[, dt])
 
     A class for representing state-space models.
 
     The StateSpace class is used to represent state-space realizations of
     linear time-invariant (LTI) systems:
 
-        dx/dt = A x + B u
-        y = C x + D u
+    .. math::
 
-    where u is the input, y is the output, and x is the state.
+          dx/dt &= A x + B u \\
+              y &= C x + D u
+
+    where `u` is the input, `y` is the output, and `x` is the state.
 
     Parameters
     ----------
@@ -217,6 +229,8 @@ class StateSpace(LTI):
     The default value of dt can be changed by changing the value of
     ``control.config.defaults['control.default_dt']``.
 
+    Note: timebase processing has moved to namedio.
+
     A state space system is callable and returns the value of the transfer
     function evaluated at a point in the complex plane.  See
     :meth:`~control.StateSpace.__call__` for a more detailed description.
@@ -244,7 +258,7 @@ class StateSpace(LTI):
     # Allow ndarray * StateSpace to give StateSpace._rmul_() priority
     __array_priority__ = 11     # override ndarray and matrix types
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, init_namedio=True, **kwargs):
         """StateSpace(A, B, C, D[, dt])
 
         Construct a state space object.
@@ -262,14 +276,27 @@ class StateSpace(LTI):
         value is read from `config.defaults['statesp.remove_useless_states']`
         (default = False).
 
+        The `init_namedio` keyword can be used to turn off initialization of
+        system and signal names.  This is used internally by the
+        :class:`LinearIOSystem` class to avoid renaming.
+
         """
-        # first get A, B, C, D matrices
+        #
+        # Process positional arguments
+        #
         if len(args) == 4:
             # The user provided A, B, C, and D matrices.
             (A, B, C, D) = args
+
         elif len(args) == 5:
             # Discrete time system
-            (A, B, C, D, _) = args
+            (A, B, C, D, dt) = args
+            if 'dt' in kwargs:
+                warn("received multiple dt arguments, "
+                     "using positional arg dt = %s" % dt)
+            kwargs['dt'] = dt
+            args = args[:-1]
+
         elif len(args) == 1:
             # Use the copy constructor.
             if not isinstance(args[0], StateSpace):
@@ -280,14 +307,10 @@ class StateSpace(LTI):
             B = args[0].B
             C = args[0].C
             D = args[0].D
-        else:
-            raise ValueError(
-                "Expected 1, 4, or 5 arguments; received %i." % len(args))
 
-        # Process keyword arguments
-        remove_useless_states = kwargs.get(
-            'remove_useless_states',
-            config.defaults['statesp.remove_useless_states'])
+        else:
+            raise TypeError(
+                "Expected 1, 4, or 5 arguments; received %i." % len(args))
 
         # Convert all matrices to standard form
         A = _ssmatrix(A)
@@ -305,46 +328,49 @@ class StateSpace(LTI):
             D = np.zeros((C.shape[0], B.shape[1]))
         D = _ssmatrix(D)
 
-        # TODO: use super here?
-        LTI.__init__(self, inputs=D.shape[1], outputs=D.shape[0])
+        # Matrices defining the linear system
         self.A = A
         self.B = B
         self.C = C
         self.D = D
 
-        # now set dt
-        if len(args) == 4:
-            if 'dt' in kwargs:
-                dt = kwargs['dt']
-            elif self._isstatic():
-                dt = None
-            else:
-                dt = config.defaults['control.default_dt']
-        elif len(args) == 5:
-            dt = args[4]
-            if 'dt' in kwargs:
-                warn("received multiple dt arguments, "
-                     "using positional arg dt = %s" % dt)
-        elif len(args) == 1:
-            try:
-                dt = args[0].dt
-            except AttributeError:
-                if self._isstatic():
-                    dt = None
-                else:
-                    dt = config.defaults['control.default_dt']
-        self.dt = dt
-        self.nstates = A.shape[1]
+        #
+        # Process keyword arguments
+        #
 
-        if 0 == self.nstates:
+        remove_useless_states = kwargs.pop(
+            'remove_useless_states',
+            config.defaults['statesp.remove_useless_states'])
+
+        # Initialize the instance variables
+        if init_namedio:
+            # Process namedio keywords
+            defaults = args[0] if len(args) == 1 else \
+                {'inputs': D.shape[1], 'outputs': D.shape[0],
+                 'states': A.shape[0]}
+            name, inputs, outputs, states, dt = _process_namedio_keywords(
+                kwargs, defaults, static=(A.size == 0), end=True)
+
+            # Initialize LTI (NamedIOSystem) object
+            super().__init__(
+                name=name, inputs=inputs, outputs=outputs,
+                states=states, dt=dt)
+        elif kwargs:
+            raise TypeError("unrecognized keyword(s): ", str(kwargs))
+
+        # Reset shapes (may not be needed once np.matrix support is removed)
+        if self._isstatic():
             # static gain
             # matrix's default "empty" shape is 1x0
             A.shape = (0, 0)
             B.shape = (0, self.ninputs)
             C.shape = (self.noutputs, 0)
 
-        # Check that the matrix sizes are consistent.
-        if self.nstates != A.shape[0]:
+        #
+        # Check to make sure everything is consistent
+        #
+        # Check that the matrix sizes are consistent
+        if A.shape[0] != A.shape[1] or self.nstates != A.shape[0]:
             raise ValueError("A must be square.")
         if self.nstates != B.shape[0]:
             raise ValueError("A and B must have the same number of rows.")
@@ -355,7 +381,10 @@ class StateSpace(LTI):
         if self.noutputs != C.shape[0]:
             raise ValueError("C and D must have the same number of rows.")
 
-        # Check for states that don't do anything, and remove them.
+        #
+        # Final processing
+        #
+        # Check for states that don't do anything, and remove them
         if remove_useless_states:
             self._remove_useless_states()
 
@@ -456,9 +485,10 @@ class StateSpace(LTI):
         self.B = delete(self.B, useless, 0)
         self.C = delete(self.C, useless, 1)
 
-        self.nstates = self.A.shape[0]
-        self.ninputs = self.B.shape[1]
-        self.noutputs = self.C.shape[0]
+        # Remove any state names that we don't need
+        self.set_states(
+            [self.state_labels[i] for i in range(self.nstates)
+             if i not in useless])
 
     def __str__(self):
         """Return string representation of the state space system."""
@@ -490,7 +520,7 @@ class StateSpace(LTI):
         s : string with LaTeX representation of model
         """
         lines = [
-            r'\[',
+            r'$$',
             (r'\left('
              + r'\begin{array}'
              + r'{' + 'rll' * self.ninputs + '}')
@@ -504,7 +534,7 @@ class StateSpace(LTI):
             r'\end{array}'
             r'\right)'
             + self._latex_dt(),
-            r'\]'])
+            r'$$'])
 
         return '\n'.join(lines)
 
@@ -522,7 +552,7 @@ class StateSpace(LTI):
             return self._latex_partitioned_stateless()
 
         lines = [
-            r'\[',
+            r'$$',
             (r'\left('
              + r'\begin{array}'
              + r'{' + 'rll' * self.nstates + '|' + 'rll' * self.ninputs + '}')
@@ -542,7 +572,7 @@ class StateSpace(LTI):
             r'\end{array}'
             + r'\right)'
             + self._latex_dt(),
-            r'\]'])
+            r'$$'])
 
         return '\n'.join(lines)
 
@@ -556,7 +586,7 @@ class StateSpace(LTI):
         s : string with LaTeX representation of model
         """
         lines = [
-            r'\[',
+            r'$$',
             r'\begin{array}{ll}',
             ]
 
@@ -586,7 +616,7 @@ class StateSpace(LTI):
         lines.extend([
             r'\end{array}'
             + self._latex_dt(),
-            r'\]'])
+            r'$$'])
 
         return '\n'.join(lines)
 
@@ -647,6 +677,13 @@ class StateSpace(LTI):
             D = self.D + other
             dt = self.dt
         else:
+            # Check to see if the right operator has priority
+            if getattr(other, '__array_priority__', None) and \
+               getattr(self, '__array_priority__', None) and \
+               other.__array_priority__ > self.__array_priority__:
+                return other.__radd__(self)
+
+            # Convert the other argument to state space
             other = _convert_to_statespace(other)
 
             # Check to make sure the dimensions are OK
@@ -697,6 +734,13 @@ class StateSpace(LTI):
             D = self.D * other
             dt = self.dt
         else:
+            # Check to see if the right operator has priority
+            if getattr(other, '__array_priority__', None) and \
+               getattr(self, '__array_priority__', None) and \
+               other.__array_priority__ > self.__array_priority__:
+                return other.__rmul__(self)
+
+            # Convert the other argument to state space
             other = _convert_to_statespace(other)
 
             # Check to make sure the dimensions are OK
@@ -750,20 +794,21 @@ class StateSpace(LTI):
             pass
         raise TypeError("can't interconnect systems")
 
-    # TODO: __div__ and __rdiv__ are not written yet.
-    def __div__(self, other):
-        """Divide two LTI systems."""
+    # TODO: general __truediv__, and  __rtruediv__; requires descriptor system support
+    def __truediv__(self, other):
+        """Division of StateSpace systems
 
-        raise NotImplementedError("StateSpace.__div__ is not implemented yet.")
+        Only division by TFs, FRDs, scalars, and arrays of scalars is
+        supported.
+        """
+        if not isinstance(other, (LTI, NamedIOSystem)):
+            return self * (1/other)
+        else:
+            return NotImplemented
 
-    def __rdiv__(self, other):
-        """Right divide two LTI systems."""
-
-        raise NotImplementedError(
-            "StateSpace.__rdiv__ is not implemented yet.")
 
     def __call__(self, x, squeeze=None, warn_infinite=True):
-        """Evaluate system's transfer function at complex frequency.
+        """Evaluate system's frequency response at complex frequencies.
 
         Returns the complex frequency response `sys(x)` where `x` is `s` for
         continuous-time systems and `z` for discrete-time systems.
@@ -925,7 +970,7 @@ class StateSpace(LTI):
 
                     # Evaluating at a pole.  Return value depends if there
                     # is a zero at the same point or not.
-                    if x_idx in self.zero():
+                    if x_idx in self.zeros():
                         out[:, :, idx] = complex(np.nan, np.nan)
                     else:
                         out[:, :, idx] = complex(np.inf, np.nan)
@@ -947,12 +992,13 @@ class StateSpace(LTI):
         return self.frequency_response(omega)
 
     # Compute poles and zeros
-    def pole(self):
+    def poles(self):
         """Compute the poles of a state space system."""
 
-        return eigvals(self.A) if self.nstates else np.array([])
+        return eigvals(self.A).astype(complex) if self.nstates \
+            else np.array([])
 
-    def zero(self):
+    def zeros(self):
         """Compute the zeros of a state space system."""
 
         if not self.nstates:
@@ -969,14 +1015,15 @@ class StateSpace(LTI):
             if nu == 0:
                 return np.array([])
             else:
+                # Use SciPy generalized eigenvalue fucntion
                 return sp.linalg.eigvals(out[8][0:nu, 0:nu],
-                                         out[9][0:nu, 0:nu])
+                                         out[9][0:nu, 0:nu]).astype(complex)
 
         except ImportError:  # Slycot unavailable. Fall back to scipy.
             if self.C.shape[0] != self.D.shape[1]:
-                raise NotImplementedError("StateSpace.zero only supports "
-                                          "systems with the same number of "
-                                          "inputs as outputs.")
+                raise NotImplementedError(
+                    "StateSpace.zero only supports systems with the same "
+                    "number of inputs as outputs.")
 
             # This implements the QZ algorithm for finding transmission zeros
             # from
@@ -994,7 +1041,7 @@ class StateSpace(LTI):
                                            (0, self.B.shape[1])), "constant")
             return np.array([x for x in sp.linalg.eigvals(L, M,
                                                           overwrite_a=True)
-                             if not isinf(x)])
+                             if not isinf(x)], dtype=complex)
 
     # Feedback around a state space system
     def feedback(self, other=1, sign=-1):
@@ -1170,8 +1217,8 @@ class StateSpace(LTI):
 
         For instance,
 
-        >>> out = ssobject.returnScipySignalLTI()
-        >>> out[3][5]
+        >>> out = ssobject.returnScipySignalLTI()               # doctest: +SKIP
+        >>> out[3][5]                                           # doctest: +SKIP
 
         is a :class:`scipy.signal.lti` object corresponding to the transfer
         function from the 6th input to the 4th output.
@@ -1252,7 +1299,8 @@ class StateSpace(LTI):
         return StateSpace(self.A, self.B[:, j], self.C[i, :],
                           self.D[i, j], self.dt)
 
-    def sample(self, Ts, method='zoh', alpha=None, prewarp_frequency=None):
+    def sample(self, Ts, method='zoh', alpha=None, prewarp_frequency=None,
+               name=None, copy_names=True, **kwargs):
         """Convert a continuous time system to discrete time
 
         Creates a discrete-time system from a continuous-time system by
@@ -1271,22 +1319,42 @@ class StateSpace(LTI):
               alpha=0)
             * backward_diff: Backwards differencing ("gbt" with alpha=1.0)
             * zoh: zero-order hold (default)
-
         alpha : float within [0, 1]
             The generalized bilinear transformation weighting parameter, which
             should only be specified with method="gbt", and is ignored
             otherwise
-
         prewarp_frequency : float within [0, infinity)
             The frequency [rad/s] at which to match with the input continuous-
             time system's magnitude and phase (the gain=1 crossover frequency,
             for example). Should only be specified with method='bilinear' or
             'gbt' with alpha=0.5 and ignored otherwise.
+        name : string, optional
+            Set the name of the sampled system.  If not specified and
+            if `copy_names` is `False`, a generic name <sys[id]> is generated
+            with a unique integer id.  If `copy_names` is `True`, the new system
+            name is determined by adding the prefix and suffix strings in
+            config.defaults['namedio.sampled_system_name_prefix'] and
+            config.defaults['namedio.sampled_system_name_suffix'], with the
+            default being to add the suffix '$sampled'.
+        copy_names : bool, Optional
+            If True, copy the names of the input signals, output
+            signals, and states to the sampled system.
 
         Returns
         -------
         sysd : StateSpace
-            Discrete time system, with sampling rate Ts
+            Discrete-time system, with sampling rate Ts
+
+        Other Parameters
+        ----------------
+        inputs : int, list of str or None, optional
+            Description of the system inputs.  If not specified, the origional
+            system inputs are used.  See :class:`InputOutputSystem` for more
+            information.
+        outputs : int, list of str or None, optional
+            Description of the system outputs.  Same format as `inputs`.
+        states : int, list of str, or None, optional
+            Description of the system states.  Same format as `inputs`.
 
         Notes
         -----
@@ -1294,21 +1362,31 @@ class StateSpace(LTI):
 
         Examples
         --------
-        >>> sys = StateSpace(0, 1, 1, 0)
-        >>> sysd = sys.sample(0.5, method='bilinear')
+        >>> G = ct.ss(0, 1, 1, 0)
+        >>> sysd = G.sample(0.5, method='bilinear')
 
         """
         if not self.isctime():
             raise ValueError("System must be continuous time system")
-
-        sys = (self.A, self.B, self.C, self.D)
-        if (method == 'bilinear' or (method == 'gbt' and alpha == 0.5)) and \
-                prewarp_frequency is not None:
-            Twarp = 2 * np.tan(prewarp_frequency * Ts/2)/prewarp_frequency
+        if prewarp_frequency is not None:
+            if method in ('bilinear', 'tustin') or \
+                    (method == 'gbt' and alpha == 0.5):
+                Twarp = 2*np.tan(prewarp_frequency*Ts/2)/prewarp_frequency
+            else:
+                warn('prewarp_frequency ignored: incompatible conversion')
+                Twarp = Ts
         else:
             Twarp = Ts
+        sys = (self.A, self.B, self.C, self.D)
         Ad, Bd, C, D, _ = cont2discrete(sys, Twarp, method, alpha)
-        return StateSpace(Ad, Bd, C, D, Ts)
+        sysd = StateSpace(Ad, Bd, C, D, Ts)
+        # copy over the system name, inputs, outputs, and states
+        if copy_names:
+            sysd._copy_names(self, prefix_suffix_name='sampled')
+            if name is not None:
+                sysd.name = name
+        # pass desired signal names if names were provided
+        return StateSpace(sysd, **kwargs)
 
     def dcgain(self, warn_infinite=False):
         """Return the zero-frequency gain
@@ -1344,7 +1422,7 @@ class StateSpace(LTI):
         """
         return self._dcgain(warn_infinite)
 
-    def dynamics(self, t, x, u=None):
+    def dynamics(self, t, x, u=None, params=None):
         """Compute the dynamics of the system
 
         Given input `u` and state `x`, returns the dynamics of the state-space
@@ -1361,7 +1439,7 @@ class StateSpace(LTI):
 
         The first argument `t` is ignored because :class:`StateSpace` systems
         are time-invariant. It is included so that the dynamics can be passed
-        to most numerical integrators, such as :func:`scipy.integrate.solve_ivp`
+        to numerical integrators, such as :func:`scipy.integrate.solve_ivp`
         and for consistency with :class:`IOSystem` systems.
 
         Parameters
@@ -1376,7 +1454,11 @@ class StateSpace(LTI):
         Returns
         -------
         dx/dt or x[t+dt] : ndarray
+
         """
+        if params is not None:
+            warn("params keyword ignored for StateSpace object")
+
         x = np.reshape(x, (-1, 1))  # force to a column in case matrix
         if np.size(x) != self.nstates:
             raise ValueError("len(x) must be equal to number of states")
@@ -1389,7 +1471,7 @@ class StateSpace(LTI):
             return (self.A @ x).reshape((-1,)) \
                 + (self.B @ u).reshape((-1,))  # return as row vector
 
-    def output(self, t, x, u=None):
+    def output(self, t, x, u=None, params=None):
         """Compute the output of the system
 
         Given input `u` and state `x`, returns the output `y` of the
@@ -1419,6 +1501,9 @@ class StateSpace(LTI):
         -------
         y : ndarray
         """
+        if params is not None:
+            warn("params keyword ignored for StateSpace object")
+
         x = np.reshape(x, (-1, 1))  # force to a column in case matrix
         if np.size(x) != self.nstates:
             raise ValueError("len(x) must be equal to number of states")
@@ -1432,36 +1517,23 @@ class StateSpace(LTI):
             return (self.C @ x).reshape((-1,)) \
                 + (self.D @ u).reshape((-1,))  # return as row vector
 
-    def _isstatic(self):
-        """True if and only if the system has no dynamics, that is,
-        if A and B are zero. """
-        return not np.any(self.A) and not np.any(self.B)
-
 
 # TODO: add discrete time check
-def _convert_to_statespace(sys, **kw):
+def _convert_to_statespace(sys, use_prefix_suffix=False):
     """Convert a system to state space form (if needed).
 
     If sys is already a state space, then it is returned.  If sys is a
     transfer function object, then it is converted to a state space and
-    returned.  If sys is a scalar, then the number of inputs and outputs can
-    be specified manually, as in:
+    returned.
 
-    >>> sys = _convert_to_statespace(3.) # Assumes inputs = outputs = 1
-    >>> sys = _convert_to_statespace(1., inputs=3, outputs=2)
+    Note: no renaming of inputs and outputs is performed; this should be done
+    by the calling function.
 
-    In the latter example, A = B = C = 0 and D = [[1., 1., 1.]
-                                                  [1., 1., 1.]].
     """
     from .xferfcn import TransferFunction
     import itertools
 
     if isinstance(sys, StateSpace):
-        if len(kw):
-            raise TypeError("If sys is a StateSpace, _convert_to_statespace "
-                            "cannot take keywords.")
-
-        # Already a state space system; just return it
         return sys
 
     elif isinstance(sys, TransferFunction):
@@ -1470,11 +1542,9 @@ def _convert_to_statespace(sys, **kw):
                [[len(num) for num in col] for col in sys.den]):
             raise ValueError("Transfer function is non-proper; can't "
                              "convert to StateSpace system.")
+
         try:
             from slycot import td04ad
-            if len(kw):
-                raise TypeError("If sys is a TransferFunction, "
-                                "_convert_to_statespace cannot take keywords.")
 
             # Change the numerator and denominator arrays so that the transfer
             # function matrix has a common denominator.
@@ -1486,10 +1556,10 @@ def _convert_to_statespace(sys, **kw):
                            denorder, den, num, tol=0)
 
             states = ssout[0]
-            return StateSpace(ssout[1][:states, :states],
-                              ssout[2][:states, :sys.ninputs],
-                              ssout[3][:sys.noutputs, :states], ssout[4],
-                              sys.dt)
+            newsys = StateSpace(
+                ssout[1][:states, :states], ssout[2][:states, :sys.ninputs],
+                ssout[3][:sys.noutputs, :states], ssout[4], sys.dt)
+
         except ImportError:
             # No Slycot.  Scipy tf->ss can't handle MIMO, but static
             # MIMO is an easy special case we can check for here
@@ -1502,7 +1572,7 @@ def _convert_to_statespace(sys, **kw):
                 for i, j in itertools.product(range(sys.noutputs),
                                               range(sys.ninputs)):
                     D[i, j] = sys.num[i][j][0] / sys.den[i][j][0]
-                return StateSpace([], [], [], D, sys.dt)
+                newsys = StateSpace([], [], [], D, sys.dt)
             else:
                 if sys.ninputs != 1 or sys.noutputs != 1:
                     raise TypeError("No support for MIMO without slycot")
@@ -1512,34 +1582,28 @@ def _convert_to_statespace(sys, **kw):
                 # the squeeze
                 A, B, C, D = \
                     sp.signal.tf2ss(squeeze(sys.num), squeeze(sys.den))
-                return StateSpace(A, B, C, D, sys.dt)
+                newsys = StateSpace(A, B, C, D, sys.dt)
 
-    elif isinstance(sys, (int, float, complex, np.number)):
-        if "inputs" in kw:
-            inputs = kw["inputs"]
-        else:
-            inputs = 1
-        if "outputs" in kw:
-            outputs = kw["outputs"]
-        else:
-            outputs = 1
+        # Copy over the signal (and system) names
+        newsys._copy_names(
+            sys,
+            prefix_suffix_name='converted' if use_prefix_suffix else None)
+        return newsys
 
-        # Generate a simple state space system of the desired dimension
-        # The following Doesn't work due to inconsistencies in ltisys:
-        #   return StateSpace([[]], [[]], [[]], eye(outputs, inputs))
-        return StateSpace([], zeros((0, inputs)), zeros((outputs, 0)),
-                          sys * ones((outputs, inputs)))
+    elif isinstance(sys, FrequencyResponseData):
+        raise TypeError("Can't convert FRD to StateSpace system.")
 
     # If this is a matrix, try to create a constant feedthrough
     try:
-        D = _ssmatrix(sys)
-        return StateSpace([], [], [], D)
+        D = _ssmatrix(np.atleast_2d(sys))
+        return StateSpace([], [], [], D, dt=None)
+
     except Exception:
         raise TypeError("Can't convert given type to StateSpace system.")
 
-
 # TODO: add discrete time option
-def _rss_generate(states, inputs, outputs, cdtype, strictly_proper=False):
+def _rss_generate(
+        states, inputs, outputs, cdtype, strictly_proper=False, name=None):
     """Generate a random state space.
 
     This does the actual random state space generation expected from rss and
@@ -1657,7 +1721,7 @@ def _rss_generate(states, inputs, outputs, cdtype, strictly_proper=False):
         ss_args = (A, B, C, D)
     else:
         ss_args = (A, B, C, D, True)
-    return StateSpace(*ss_args)
+    return StateSpace(*ss_args, name=name)
 
 
 # Convert a MIMO system to a SISO system
@@ -1714,7 +1778,9 @@ def _mimo2siso(sys, input, output, warn_conversion=False):
         new_B = sys.B[:, input]
         new_C = sys.C[output, :]
         new_D = sys.D[output, input]
-        sys = StateSpace(sys.A, new_B, new_C, new_D, sys.dt)
+        sys = StateSpace(sys.A, new_B, new_C, new_D, sys.dt,
+                         name=sys.name,
+                         inputs=sys.input_labels[input], outputs=sys.output_labels[output])
 
     return sys
 
@@ -1763,104 +1829,14 @@ def _mimo2simo(sys, input, warn_conversion=False):
         #  Y = C*X + D*U
         new_B = sys.B[:, input:input+1]
         new_D = sys.D[:, input:input+1]
-        sys = StateSpace(sys.A, new_B, sys.C, new_D, sys.dt)
+        sys = StateSpace(sys.A, new_B, sys.C, new_D, sys.dt,
+                         name=sys.name,
+                         inputs=sys.input_labels[input], outputs=sys.output_labels)
 
     return sys
 
 
-def ss(*args, **kwargs):
-    """ss(A, B, C, D[, dt])
-
-    Create a state space system.
-
-    The function accepts either 1, 4 or 5 parameters:
-
-    ``ss(sys)``
-        Convert a linear system into space system form. Always creates a
-        new system, even if sys is already a StateSpace object.
-
-    ``ss(A, B, C, D)``
-        Create a state space system from the matrices of its state and
-        output equations:
-
-        .. math::
-            \\dot x = A \\cdot x + B \\cdot u
-
-            y = C \\cdot x + D \\cdot u
-
-    ``ss(A, B, C, D, dt)``
-        Create a discrete-time state space system from the matrices of
-        its state and output equations:
-
-        .. math::
-            x[k+1] = A \\cdot x[k] + B \\cdot u[k]
-
-            y[k] = C \\cdot x[k] + D \\cdot u[ki]
-
-        The matrices can be given as *array like* data types or strings.
-        Everything that the constructor of :class:`numpy.matrix` accepts is
-        permissible here too.
-
-    Parameters
-    ----------
-    sys: StateSpace or TransferFunction
-        A linear system
-    A: array_like or string
-        System matrix
-    B: array_like or string
-        Control matrix
-    C: array_like or string
-        Output matrix
-    D: array_like or string
-        Feed forward matrix
-    dt: If present, specifies the timebase of the system
-
-    Returns
-    -------
-    out: :class:`StateSpace`
-        The new linear system
-
-    Raises
-    ------
-    ValueError
-        if matrix sizes are not self-consistent
-
-    See Also
-    --------
-    StateSpace
-    tf
-    ss2tf
-    tf2ss
-
-    Examples
-    --------
-    >>> # Create a StateSpace object from four "matrices".
-    >>> sys1 = ss("1. -2; 3. -4", "5.; 7", "6. 8", "9.")
-
-    >>> # Convert a TransferFunction to a StateSpace object.
-    >>> sys_tf = tf([2.], [1., 3])
-    >>> sys2 = ss(sys_tf)
-
-    """
-
-    if len(args) == 4 or len(args) == 5:
-        return StateSpace(*args, **kwargs)
-    elif len(args) == 1:
-        from .xferfcn import TransferFunction
-        sys = args[0]
-        if isinstance(sys, StateSpace):
-            return deepcopy(sys)
-        elif isinstance(sys, TransferFunction):
-            return tf2ss(sys)
-        else:
-            raise TypeError("ss(sys): sys must be a StateSpace or "
-                            "TransferFunction object.  It is %s." % type(sys))
-    else:
-        raise ValueError(
-            "Needs 1, 4, or 5 arguments; received %i." % len(args))
-
-
-def tf2ss(*args):
+def tf2ss(*args, **kwargs):
     """tf2ss(sys)
 
     Transform a transfer function to a state space system.
@@ -1868,11 +1844,11 @@ def tf2ss(*args):
     The function accepts either 1 or 2 parameters:
 
     ``tf2ss(sys)``
-        Convert a linear system into transfer function form. Always creates
-        a new system, even if sys is already a TransferFunction object.
+        Convert a linear system into space space form. Always creates
+        a new system, even if sys is already a StateSpace object.
 
     ``tf2ss(num, den)``
-        Create a transfer function system from its numerator and denominator
+        Create a state space system from its numerator and denominator
         polynomial coefficients.
 
         For details see: :func:`tf`
@@ -1891,6 +1867,16 @@ def tf2ss(*args):
     out : StateSpace
         New linear system in state space form
 
+    Other Parameters
+    ----------------
+    inputs, outputs : str, or list of str, optional
+        List of strings that name the individual signals of the transformed
+        system.  If not given, the inputs and outputs are the same as the
+        original system.
+    name : string, optional
+        System name. If unspecified, a generic name <sys[id]> is generated
+        with a unique integer id.
+
     Raises
     ------
     ValueError
@@ -1910,111 +1896,33 @@ def tf2ss(*args):
     --------
     >>> num = [[[1., 2.], [3., 4.]], [[5., 6.], [7., 8.]]]
     >>> den = [[[9., 8., 7.], [6., 5., 4.]], [[3., 2., 1.], [-1., -2., -3.]]]
-    >>> sys1 = tf2ss(num, den)
+    >>> sys1 = ct.tf2ss(num, den)
 
-    >>> sys_tf = tf(num, den)
-    >>> sys2 = tf2ss(sys_tf)
+    >>> sys_tf = ct.tf(num, den)
+    >>> sys2 = ct.tf2ss(sys_tf)
 
     """
 
     from .xferfcn import TransferFunction
     if len(args) == 2 or len(args) == 3:
         # Assume we were given the num, den
-        return _convert_to_statespace(TransferFunction(*args))
+        return StateSpace(
+            _convert_to_statespace(TransferFunction(*args)), **kwargs)
 
     elif len(args) == 1:
         sys = args[0]
         if not isinstance(sys, TransferFunction):
             raise TypeError("tf2ss(sys): sys must be a TransferFunction "
                             "object.")
-        return _convert_to_statespace(sys)
+        return StateSpace(
+            _convert_to_statespace(
+                sys,
+                use_prefix_suffix=not sys._generic_name_check()),
+            **kwargs)
     else:
         raise ValueError("Needs 1 or 2 arguments; received %i." % len(args))
 
 
-def rss(states=1, outputs=1, inputs=1, strictly_proper=False):
-    """
-    Create a stable *continuous* random state space object.
-
-    Parameters
-    ----------
-    states : int
-        Number of state variables
-    outputs : int
-        Number of system outputs
-    inputs : int
-        Number of system inputs
-    strictly_proper : bool, optional
-        If set to 'True', returns a proper system (no direct term).
-
-    Returns
-    -------
-    sys : StateSpace
-        The randomly created linear system
-
-    Raises
-    ------
-    ValueError
-        if any input is not a positive integer
-
-    See Also
-    --------
-    drss
-
-    Notes
-    -----
-    If the number of states, inputs, or outputs is not specified, then the
-    missing numbers are assumed to be 1.  The poles of the returned system
-    will always have a negative real part.
-
-    """
-
-    return _rss_generate(states, inputs, outputs, 'c',
-                         strictly_proper=strictly_proper)
-
-
-def drss(states=1, outputs=1, inputs=1, strictly_proper=False):
-    """
-    Create a stable *discrete* random state space object.
-
-    Parameters
-    ----------
-    states : int
-        Number of state variables
-    inputs : integer
-        Number of system inputs
-    outputs : int
-        Number of system outputs
-    strictly_proper: bool, optional
-        If set to 'True', returns a proper system (no direct term).
-
-
-    Returns
-    -------
-    sys : StateSpace
-        The randomly created linear system
-
-    Raises
-    ------
-    ValueError
-        if any input is not a positive integer
-
-    See Also
-    --------
-    rss
-
-    Notes
-    -----
-    If the number of states, inputs, or outputs is not specified, then the
-    missing numbers are assumed to be 1.  The poles of the returned system
-    will always have a magnitude less than 1.
-
-    """
-
-    return _rss_generate(states, inputs, outputs, 'd',
-                         strictly_proper=strictly_proper)
-
-
 def ssdata(sys):
     """
     Return state space data objects for a system
@@ -2031,3 +1939,61 @@ def ssdata(sys):
     """
     ss = _convert_to_statespace(sys)
     return ss.A, ss.B, ss.C, ss.D
+
+
+def linfnorm(sys, tol=1e-10):
+    """L-infinity norm of a linear system
+
+    Parameters
+    ----------
+    sys : LTI (StateSpace or TransferFunction)
+      system to evalute L-infinity norm of
+    tol : real scalar
+      tolerance on norm estimate
+
+    Returns
+    -------
+    gpeak : non-negative scalar
+      L-infinity norm
+    fpeak : non-negative scalar
+      Frequency, in rad/s, at which gpeak occurs
+
+    For stable systems, the L-infinity and H-infinity norms are equal;
+    for unstable systems, the H-infinity norm is infinite, while the
+    L-infinity norm is finite if the system has no poles on the
+    imaginary axis.
+
+    See also
+    --------
+    slycot.ab13dd : the Slycot routine linfnorm that does the calculation
+    """
+
+    if ab13dd is None:
+        raise ControlSlycot("Can't find slycot module 'ab13dd'")
+
+    a, b, c, d = ssdata(_convert_to_statespace(sys))
+    e = np.eye(a.shape[0])
+
+    n = a.shape[0]
+    m = b.shape[1]
+    p = c.shape[0]
+
+    if n == 0:
+        # ab13dd doesn't accept empty A, B, C, D;
+        # static gain case is easy enough to compute
+        gpeak = scipy.linalg.svdvals(d)[0]
+        # max svd is constant with freq; arbitrarily choose 0 as peak
+        fpeak = 0
+        return gpeak, fpeak
+
+    dico = 'C' if sys.isctime() else 'D'
+    jobe = 'I'
+    equil = 'S'
+    jobd = 'Z' if all(0 == d.flat) else 'D'
+
+    gpeak, fpeak = ab13dd(dico, jobe, equil, jobd, n, m, p, a, e, b, c, d, tol)
+
+    if dico=='D':
+        fpeak /= sys.dt
+
+    return gpeak, fpeak
