@@ -81,8 +81,9 @@ def test_nyquist_basic(ss_siso):
         tf_siso, plot=False, return_contour=True, omega_num=20)
     assert len(contour) == 20
 
-    count, contour = nyquist_plot(
-        tf_siso, plot=False, omega_limits=(1, 100), return_contour=True)
+    with pytest.warns(UserWarning, match="encirclements was a non-integer"):
+        count, contour = nyquist_plot(
+            tf_siso, plot=False, omega_limits=(1, 100), return_contour=True)
     assert_allclose(contour[0], 1j)
     assert_allclose(contour[-1], 100j)
 
@@ -374,6 +375,18 @@ def test_phase_wrap(TF, wrap_phase, min_phase, max_phase):
     assert(max(phase) <= max_phase)
 
 
+def test_phase_wrap_multiple_systems():
+    sys_unstable = ctrl.zpk([],[1,1], gain=1)
+
+    mag, phase, omega = ctrl.bode(sys_unstable, plot=False)
+    assert(np.min(phase) >= -2*np.pi)
+    assert(np.max(phase) <= -1*np.pi)
+
+    mag, phase, omega = ctrl.bode((sys_unstable, sys_unstable), plot=False)
+    assert(np.min(phase) >= -2*np.pi)
+    assert(np.max(phase) <= -1*np.pi)
+
+
 def test_freqresp_warn_infinite():
     """Test evaluation warnings for transfer functions w/ pole at the origin"""
     sys_finite = ctrl.tf([1], [1, 0.01])
@@ -418,11 +431,11 @@ def test_dcgain_consistency():
     """Test to make sure that DC gain is consistently evaluated"""
     # Set up transfer function with pole at the origin
     sys_tf = ctrl.tf([1], [1, 0])
-    assert 0 in sys_tf.pole()
+    assert 0 in sys_tf.poles()
 
     # Set up state space system with pole at the origin
     sys_ss = ctrl.tf2ss(sys_tf)
-    assert 0 in sys_ss.pole()
+    assert 0 in sys_ss.poles()
 
     # Finite (real) numerator over 0 denominator => inf + nanj
     np.testing.assert_equal(
@@ -440,8 +453,8 @@ def test_dcgain_consistency():
 
     # Set up transfer function with pole, zero at the origin
     sys_tf = ctrl.tf([1, 0], [1, 0])
-    assert 0 in sys_tf.pole()
-    assert 0 in sys_tf.zero()
+    assert 0 in sys_tf.poles()
+    assert 0 in sys_tf.zeros()
 
     # Pole and zero at the origin should give nan + nanj for the response
     np.testing.assert_equal(
@@ -456,7 +469,7 @@ def test_dcgain_consistency():
         ctrl.tf2ss(ctrl.tf([1], [1, 0]))
 
     # Different systems give different representations => test accordingly
-    if 0 in sys_ss.pole() and 0 in sys_ss.zero():
+    if 0 in sys_ss.poles() and 0 in sys_ss.zeros():
         # Pole and zero at the origin => should get (nan + nanj)
         np.testing.assert_equal(
             sys_ss(0, warn_infinite=False), complex(np.nan, np.nan))
@@ -464,7 +477,7 @@ def test_dcgain_consistency():
             sys_ss(0j, warn_infinite=False), complex(np.nan, np.nan))
         np.testing.assert_equal(
             sys_ss.dcgain(), np.nan)
-    elif 0 in sys_ss.pole():
+    elif 0 in sys_ss.poles():
         # Pole at the origin, but zero elsewhere => should get (inf + nanj)
         np.testing.assert_equal(
             sys_ss(0, warn_infinite=False), complex(np.inf, np.nan))
@@ -479,11 +492,11 @@ def test_dcgain_consistency():
     # Pole with non-zero, complex numerator => inf + infj
     s = ctrl.tf('s')
     sys_tf = (s + 1) / (s**2 + 1)
-    assert 1j in sys_tf.pole()
+    assert 1j in sys_tf.poles()
 
     # Set up state space system with pole on imaginary axis
     sys_ss = ctrl.tf2ss(sys_tf)
-    assert 1j in sys_tf.pole()
+    assert 1j in sys_tf.poles()
 
     # Make sure we get correct response if evaluated at the pole
     np.testing.assert_equal(
