@@ -9,13 +9,16 @@ created for that purpose.
 """
 
 import re
+import warnings
+import pytest
 
 import numpy as np
-import pytest
+from math import sqrt
 
 import control as ct
 from control import iosys as ios
-from control.tests.conftest import noscipy0, matrixfilter
+from control.tests.conftest import matrixfilter
+
 
 class TestIOSys:
 
@@ -39,6 +42,9 @@ class TestIOSys:
             [[-1, 1], [0, -2]], [[0, 1], [1, 0]],
             [[1, 0], [0, 1]], np.zeros((2, 2)))
 
+        # Create a static gain linear system
+        T.staticgain = ct.StateSpace([], [], [], 1)
+
         # Create simulation parameters
         T.T = np.linspace(0, 10, 100)
         T.U = np.sin(T.T)
@@ -46,11 +52,10 @@ class TestIOSys:
 
         return T
 
-    @noscipy0
     def test_linear_iosys(self, tsys):
         # Create an input/output system from the linear system
         linsys = tsys.siso_linsys
-        iosys = ios.LinearIOSystem(linsys)
+        iosys = ios.LinearIOSystem(linsys).copy()
 
         # Make sure that the right hand side matches linear system
         for x, u in (([0, 0], 0), ([1, 0], 0), ([0, 1], 0), ([0, 0], 1)):
@@ -65,7 +70,11 @@ class TestIOSys:
         np.testing.assert_array_almost_equal(lti_t, ios_t)
         np.testing.assert_allclose(lti_y, ios_y, atol=0.002, rtol=0.)
 
-    @noscipy0
+        # Make sure that a static linear system has dt=None
+        # and otherwise dt is as specified
+        assert ios.LinearIOSystem(tsys.staticgain).dt is None
+        assert ios.LinearIOSystem(tsys.staticgain, dt=.1).dt == .1
+
     def test_tf2io(self, tsys):
         # Create a transfer function from the state space system
         linsys = tsys.siso_linsys
@@ -129,8 +138,8 @@ class TestIOSys:
         ios_linearized = ios.linearize(ios_unspecified, [0, 0], [0])
         print(ios_linearized)
 
-    @noscipy0
-    def test_nonlinear_iosys(self, tsys):
+    @pytest.mark.parametrize("ss", [ios.NonlinearIOSystem, ct.ss])
+    def test_nonlinear_iosys(self, tsys, ss):
         # Create a simple nonlinear I/O system
         nlsys = ios.NonlinearIOSystem(predprey)
         T = tsys.T
@@ -208,7 +217,7 @@ class TestIOSys:
     @pytest.mark.usefixtures("editsdefaults")
     def test_linearize_named_signals(self, kincar):
         # Full form of the call
-        linearized = kincar.linearize([0, 0, 0], [0, 0], copy=True,
+        linearized = kincar.linearize([0, 0, 0], [0, 0], copy_names=True,
                                       name='linearized')
         assert linearized.name == 'linearized'
         assert linearized.find_input('v') == 0
@@ -220,28 +229,41 @@ class TestIOSys:
         assert linearized.find_state('theta') == 2
 
         # If we copy signal names w/out a system name, append '$linearized'
-        linearized = kincar.linearize([0, 0, 0], [0, 0], copy=True)
+        linearized = kincar.linearize([0, 0, 0], [0, 0], copy_names=True)
         assert linearized.name == kincar.name + '$linearized'
 
-        # Test legacy version as well
-        ct.use_legacy_defaults('0.8.4')
-        ct.config.use_numpy_matrix(False)       # np.matrix deprecated
-        linearized = kincar.linearize([0, 0, 0], [0, 0], copy=True)
-        assert linearized.name == kincar.name + '_linearized'
-
         # If copy is False, signal names should not be copied
-        lin_nocopy = kincar.linearize(0, 0, copy=False)
+        lin_nocopy = kincar.linearize(0, 0, copy_names=False)
         assert lin_nocopy.find_input('v') is None
         assert lin_nocopy.find_output('x') is None
         assert lin_nocopy.find_state('x') is None
 
-    @noscipy0
+        # if signal names are provided, they should override those of kincar
+        linearized_newnames = kincar.linearize([0, 0, 0], [0, 0],
+            name='linearized',
+            copy_names=True, inputs=['v2', 'phi2'], outputs=['x2','y2'])
+        assert linearized_newnames.name == 'linearized'
+        assert linearized_newnames.find_input('v2') == 0
+        assert linearized_newnames.find_input('phi2') == 1
+        assert linearized_newnames.find_input('v') is None
+        assert linearized_newnames.find_input('phi') is None
+        assert linearized_newnames.find_output('x2') == 0
+        assert linearized_newnames.find_output('y2') == 1
+        assert linearized_newnames.find_output('x') is None
+        assert linearized_newnames.find_output('y') is None
+
+        # Test legacy version as well
+        ct.use_legacy_defaults('0.8.4')
+        ct.config.use_numpy_matrix(False)       # np.matrix deprecated
+        linearized = kincar.linearize([0, 0, 0], [0, 0], copy_names=True)
+        assert linearized.name == kincar.name + '_linearized'
+
     def test_connect(self, tsys):
         # Define a couple of (linear) systems to interconnection
         linsys1 = tsys.siso_linsys
-        iosys1 = ios.LinearIOSystem(linsys1)
+        iosys1 = ios.LinearIOSystem(linsys1, name='iosys1')
         linsys2 = tsys.siso_linsys
-        iosys2 = ios.LinearIOSystem(linsys2)
+        iosys2 = ios.LinearIOSystem(linsys2, name='iosys2')
 
         # Connect systems in different ways and compare to StateSpace
         linsys_series = linsys2 * linsys1
@@ -293,7 +315,6 @@ class TestIOSys:
         np.testing.assert_array_almost_equal(lti_t, ios_t)
         np.testing.assert_allclose(lti_y, ios_y,atol=0.002,rtol=0.)
 
-    @noscipy0
     @pytest.mark.parametrize(
         "connections, inplist, outlist",
         [pytest.param([[(1, 0), (0, 0, 1)]], [[(0, 0, 1)]], [[(1, 0, 1)]],
@@ -337,7 +358,6 @@ class TestIOSys:
         np.testing.assert_array_almost_equal(lti_t, ios_t)
         np.testing.assert_allclose(lti_y, ios_y, atol=0.002, rtol=0.)
 
-    @noscipy0
     @pytest.mark.parametrize(
         "connections, inplist, outlist",
         [pytest.param([['sys2.u[0]', 'sys1.y[0]']],
@@ -374,7 +394,6 @@ class TestIOSys:
         np.testing.assert_array_almost_equal(lti_t, ios_t)
         np.testing.assert_allclose(lti_y, ios_y, atol=0.002, rtol=0.)
 
-    @noscipy0
     def test_static_nonlinearity(self, tsys):
         # Linear dynamical system
         linsys = tsys.siso_linsys
@@ -399,7 +418,6 @@ class TestIOSys:
         np.testing.assert_array_almost_equal(lti_y, ios_y, decimal=2)
 
 
-    @noscipy0
     @pytest.mark.filterwarnings("ignore:Duplicate name::control.iosys")
     def test_algebraic_loop(self, tsys):
         # Create some linear and nonlinear systems to play with
@@ -407,8 +425,8 @@ class TestIOSys:
         lnios = ios.LinearIOSystem(linsys)
         nlios =  ios.NonlinearIOSystem(None, \
             lambda t, x, u, params: u*u, inputs=1, outputs=1)
-        nlios1 = nlios.copy()
-        nlios2 = nlios.copy()
+        nlios1 = nlios.copy(name='nlios1')
+        nlios2 = nlios.copy(name='nlios2')
 
         # Set up parameters for simulation
         T, U, X0 = tsys.T, tsys.U, tsys.X0
@@ -469,12 +487,11 @@ class TestIOSys:
         with pytest.raises(RuntimeError):
             ios.input_output_response(*args)
 
-    @noscipy0
     def test_summer(self, tsys):
         # Construct a MIMO system for testing
         linsys = tsys.mimo_linsys1
-        linio1 = ios.LinearIOSystem(linsys)
-        linio2 = ios.LinearIOSystem(linsys)
+        linio1 = ios.LinearIOSystem(linsys, name='linio1')
+        linio2 = ios.LinearIOSystem(linsys, name='linio2')
 
         linsys_parallel = linsys + linsys
         iosys_parallel = linio1 + linio2
@@ -488,7 +505,6 @@ class TestIOSys:
         ios_t, ios_y = ios.input_output_response(iosys_parallel, T, U, X0)
         np.testing.assert_allclose(ios_y, lin_y,atol=0.002,rtol=0.)
 
-    @noscipy0
     def test_rmul(self, tsys):
         # Test right multiplication
         # TODO: replace with better tests when conversions are implemented
@@ -509,7 +525,6 @@ class TestIOSys:
         lti_t, lti_y = ct.forced_response(ioslin, T, U*U, X0)
         np.testing.assert_array_almost_equal(ios_y, lti_y*lti_y, decimal=3)
 
-    @noscipy0
     def test_neg(self, tsys):
         """Test negation of a system"""
 
@@ -532,7 +547,6 @@ class TestIOSys:
         lti_t, lti_y = ct.forced_response(ioslin, T, U*U, X0)
         np.testing.assert_array_almost_equal(ios_y, -lti_y, decimal=3)
 
-    @noscipy0
     def test_feedback(self, tsys):
         # Set up parameters for simulation
         T, U, X0 = tsys.T, tsys.U, tsys.X0
@@ -548,7 +562,6 @@ class TestIOSys:
         lti_t, lti_y = ct.forced_response(linsys, T, U, X0)
         np.testing.assert_allclose(ios_y, lti_y,atol=0.002,rtol=0.)
 
-    @noscipy0
     def test_bdalg_functions(self, tsys):
         """Test block diagram functions algebra on I/O systems"""
         # Set up parameters for simulation
@@ -595,7 +608,6 @@ class TestIOSys:
         ios_t, ios_y = ios.input_output_response(iosys_feedback, T, U, X0)
         np.testing.assert_allclose(ios_y, lin_y,atol=0.002,rtol=0.)
 
-    @noscipy0
     def test_algebraic_functions(self, tsys):
         """Test algebraic operations on I/O systems"""
         # Set up parameters for simulation
@@ -647,7 +659,6 @@ class TestIOSys:
         ios_t, ios_y = ios.input_output_response(iosys_negate, T, U, X0)
         np.testing.assert_allclose(ios_y, lin_y,atol=0.002,rtol=0.)
 
-    @noscipy0
     def test_nonsquare_bdalg(self, tsys):
         # Set up parameters for simulation
         T = tsys.T
@@ -698,7 +709,6 @@ class TestIOSys:
         with pytest.raises(ValueError):
             ct.series(*args)
 
-    @noscipy0
     def test_discrete(self, tsys):
         """Test discrete time functionality"""
         # Create some linear and nonlinear systems to play with
@@ -731,8 +741,34 @@ class TestIOSys:
         np.testing.assert_allclose(ios_t, lin_t,atol=0.002,rtol=0.)
         np.testing.assert_allclose(ios_y, lin_y,atol=0.002,rtol=0.)
 
-    def test_find_eqpts(self, tsys):
-        """Test find_eqpt function"""
+    def test_discrete_iosys(self, tsys):
+        """Create a discrete time system from scratch"""
+        linsys = ct.StateSpace(
+            [[-1, 1], [0, -2]], [[0], [1]], [[1, 0]], [[0]], True)
+
+        # Create nonlinear version of the same system
+        def nlsys_update(t, x, u, params):
+            A, B = params['A'], params['B']
+            return A @ x + B @ u
+        def nlsys_output(t, x, u, params):
+            C = params['C']
+            return C @ x
+        nlsys = ct.NonlinearIOSystem(
+            nlsys_update, nlsys_output, inputs=1, outputs=1, states=2, dt=True)
+
+        # Set up parameters for simulation
+        T, U, X0 = tsys.T, tsys.U, tsys.X0
+
+        # Simulate and compare to LTI output
+        ios_t, ios_y = ios.input_output_response(
+            nlsys, T, U, X0,
+            params={'A': linsys.A, 'B': linsys.B, 'C': linsys.C})
+        lin_t, lin_y = ct.forced_response(linsys, T, U, X0)
+        np.testing.assert_allclose(ios_t, lin_t,atol=0.002,rtol=0.)
+        np.testing.assert_allclose(ios_y, lin_y,atol=0.002,rtol=0.)
+
+    def test_find_eqpts_dfan(self, tsys):
+        """Test find_eqpt function on dfan example"""
         # Simple equilibrium point with no inputs
         nlsys = ios.NonlinearIOSystem(predprey)
         xeq, ueq, result = ios.find_eqpt(
@@ -801,6 +837,17 @@ class TestIOSys:
         np.testing.assert_array_almost_equal(
             nlsys_full._rhs(0, xeq, ueq)[-4:], np.zeros((4,)), decimal=5)
 
+        # Same test as before, but now all constraints are in the state vector
+        nlsys_full = ios.NonlinearIOSystem(pvtol_full, None)
+        xeq, ueq, result = ios.find_eqpt(
+            nlsys_full, [0, 0, 0.1, 0.1, 0, 0], [0.01, 4*9.8],
+            idx=[2, 3, 4, 5], ix=[0, 1, 2, 3], return_result=True)
+        assert result.success
+        np.testing.assert_array_almost_equal(
+            nlsys_full._out(0, xeq, ueq)[[2, 3]], [0.1, 0.1], decimal=5)
+        np.testing.assert_array_almost_equal(
+            nlsys_full._rhs(0, xeq, ueq)[-4:], np.zeros((4,)), decimal=5)
+
         # Fix one input and vary the other
         nlsys_full = ios.NonlinearIOSystem(pvtol_full, None)
         xeq, ueq, result = ios.find_eqpt(
@@ -841,7 +888,6 @@ class TestIOSys:
         assert xeq is None
         assert ueq is None
 
-    @noscipy0
     def test_params(self, tsys):
         # Start with the default set of parameters
         ios_secord_default = ios.NonlinearIOSystem(
@@ -1016,8 +1062,12 @@ class TestIOSys:
 
         ct.config.use_legacy_defaults('0.8.4')  # changed delims in 0.9.0
         ct.config.use_numpy_matrix(False)       # np.matrix deprecated
-        ct.InputOutputSystem._idCounter = 0
-        sys = ct.LinearIOSystem(tsys.mimo_linsys1)
+
+        # Create a system with a known ID
+        ct.namedio.NamedIOSystem._idCounter = 0
+        sys = ct.ss(
+            tsys.mimo_linsys1.A, tsys.mimo_linsys1.B,
+            tsys.mimo_linsys1.C, tsys.mimo_linsys1.D)
 
         assert sys.name == "sys[0]"
         assert sys.copy().name == "copy of sys[0]"
@@ -1067,7 +1117,7 @@ class TestIOSys:
 
         # Same system conflict
         with pytest.warns(UserWarning):
-            unnamedsys1 * unnamedsys1
+            namedsys * namedsys
 
     @pytest.mark.usefixtures("editsdefaults")
     def test_signals_naming_convention_0_8_4(self, tsys):
@@ -1080,8 +1130,13 @@ class TestIOSys:
 
         ct.config.use_legacy_defaults('0.8.4')  # changed delims in 0.9.0
         ct.config.use_numpy_matrix(False)       # np.matrix deprecated
-        ct.InputOutputSystem._idCounter = 0
-        sys = ct.LinearIOSystem(tsys.mimo_linsys1)
+
+        # Create a system with a known ID
+        ct.namedio.NamedIOSystem._idCounter = 0
+        sys = ct.ss(
+            tsys.mimo_linsys1.A, tsys.mimo_linsys1.B,
+            tsys.mimo_linsys1.C, tsys.mimo_linsys1.D)
+
         for statename in ["x[0]", "x[1]"]:
             assert statename in sys.state_index
         for inputname in ["u[0]", "u[1]"]:
@@ -1128,12 +1183,12 @@ class TestIOSys:
 
         # Same system conflict
         with pytest.warns(UserWarning):
-            same_name_series = unnamedsys * unnamedsys
-            assert "sys[1].x[0]" in same_name_series.state_index
-            assert "copy of sys[1].x[0]" in same_name_series.state_index
+            same_name_series = namedsys * namedsys
+            assert "namedsys.x0" in same_name_series.state_index
+            assert "copy of namedsys.x0" in same_name_series.state_index
 
     def test_named_signals_linearize_inconsistent(self, tsys):
-        """Mare sure that providing inputs or outputs not consistent with
+        """Make sure that providing inputs or outputs not consistent with
            updfcn or outfcn fail
         """
 
@@ -1178,15 +1233,26 @@ class TestIOSys:
             with pytest.raises(ValueError):
                 sys2.linearize(x0, u0)
 
+    def test_linearize_concatenation(self, kincar):
+        # Create a simple nonlinear system to check (kinematic car)
+        iosys = kincar
+        linearized = iosys.linearize([0, np.array([0, 0])], [0, 0])
+        np.testing.assert_array_almost_equal(linearized.A, np.zeros((3,3)))
+        np.testing.assert_array_almost_equal(
+            linearized.B, [[1, 0], [0, 0], [0, 1]])
+        np.testing.assert_array_almost_equal(
+            linearized.C, [[1, 0, 0], [0, 1, 0]])
+        np.testing.assert_array_almost_equal(linearized.D, np.zeros((2,2)))
+
     def test_lineariosys_statespace(self, tsys):
         """Make sure that a LinearIOSystem is also a StateSpace object"""
-        iosys_siso = ct.LinearIOSystem(tsys.siso_linsys)
-        iosys_siso2 = ct.LinearIOSystem(tsys.siso_linsys)
+        iosys_siso = ct.LinearIOSystem(tsys.siso_linsys, name='siso')
+        iosys_siso2 = ct.LinearIOSystem(tsys.siso_linsys, name='siso2')
         assert isinstance(iosys_siso, ct.StateSpace)
 
         # Make sure that state space functions work for LinearIOSystems
         np.testing.assert_allclose(
-            iosys_siso.pole(), tsys.siso_linsys.pole())
+            iosys_siso.poles(), tsys.siso_linsys.poles())
         omega = np.logspace(.1, 10, 100)
         mag_io, phase_io, omega_io = iosys_siso.frequency_response(omega)
         mag_ss, phase_ss, omega_ss = tsys.siso_linsys.frequency_response(omega)
@@ -1364,7 +1430,7 @@ class TestIOSys:
                                       name="sys")
 
         # Duplicate objects
-        with pytest.warns(UserWarning, match="Duplicate object"):
+        with pytest.warns(UserWarning, match="duplicate object"):
             ios_series = nlios * nlios
 
         # Nonduplicate objects
@@ -1372,7 +1438,7 @@ class TestIOSys:
         ct.config.use_numpy_matrix(False)       # np.matrix deprecated
         nlios1 = nlios.copy()
         nlios2 = nlios.copy()
-        with pytest.warns(UserWarning, match="Duplicate name"):
+        with pytest.warns(UserWarning, match="duplicate name"):
             ios_series = nlios1 * nlios2
             assert "copy of sys_1.x[0]" in ios_series.state_index.keys()
             assert "copy of sys.x[0]" in ios_series.state_index.keys()
@@ -1386,7 +1452,7 @@ class TestIOSys:
                                        lambda t, x, u, params: u * u,
                                        inputs=1, outputs=1, name="sys")
 
-        with pytest.warns(UserWarning, match="Duplicate name"):
+        with pytest.warns(UserWarning, match="duplicate name"):
             ct.InterconnectedSystem([nlios1, iosys_siso, nlios2],
                                     inputs=0, outputs=0, states=0)
 
@@ -1397,11 +1463,10 @@ class TestIOSys:
         nlios2 = ios.NonlinearIOSystem(None,
                                        lambda t, x, u, params: u * u,
                                        inputs=1, outputs=1, name="nlios2")
-        with pytest.warns(None) as record:
+        with warnings.catch_warnings():
+            warnings.simplefilter("error")
             ct.InterconnectedSystem([nlios1, iosys_siso, nlios2],
                                     inputs=0, outputs=0, states=0)
-        if record:
-            pytest.fail("Warning not expected: " + record[0].message)
 
 
 def test_linear_interconnection():
@@ -1426,6 +1491,11 @@ def test_linear_interconnection():
         inputs = ('u[0]', 'u[1]'),
         outputs = ('y[0]', 'y[1]'),
         name = 'sys2')
+    tf_siso = ct.tf(1, [0.1, 1])
+    ss_siso = ct.ss(1, 2, 1, 1)
+    nl_siso = ios.NonlinearIOSystem(
+        lambda t, x, u, params: x*x,
+        lambda t, x, u, params: u*x, states=1, inputs=1, outputs=1)
 
     # Create a "regular" InterconnectedSystem
     nl_connect = ios.interconnect(
@@ -1472,6 +1542,18 @@ def test_linear_interconnection():
     np.testing.assert_array_almost_equal(io_connect.C, ss_connect.C)
     np.testing.assert_array_almost_equal(io_connect.D, ss_connect.D)
 
+    # make sure interconnections of linear systems are linear and
+    # if a nonlinear system is included then system is nonlinear
+    assert isinstance(ss_siso*ss_siso, ios.LinearIOSystem)
+    assert isinstance(tf_siso*ss_siso, ios.LinearIOSystem)
+    assert isinstance(ss_siso*tf_siso, ios.LinearIOSystem)
+    assert ~isinstance(ss_siso*nl_siso, ios.LinearIOSystem)
+    assert ~isinstance(nl_siso*ss_siso, ios.LinearIOSystem)
+    assert ~isinstance(nl_siso*nl_siso, ios.LinearIOSystem)
+    assert ~isinstance(tf_siso*nl_siso, ios.LinearIOSystem)
+    assert ~isinstance(nl_siso*tf_siso, ios.LinearIOSystem)
+    assert ~isinstance(nl_siso*nl_siso, ios.LinearIOSystem)
+
 
 def predprey(t, x, u, params={}):
     """Predator prey dynamics"""
@@ -1491,7 +1573,7 @@ def predprey(t, x, u, params={}):
 
 def pvtol(t, x, u, params={}):
     """Reduced planar vertical takeoff and landing dynamics"""
-    from math import sin, cos
+    from math import cos, sin
     m = params.get('m', 4.)      # kg, system mass
     J = params.get('J', 0.0475)  # kg m^2, system inertia
     r = params.get('r', 0.25)    # m, thrust offset
@@ -1507,7 +1589,7 @@ def pvtol(t, x, u, params={}):
 
 
 def pvtol_full(t, x, u, params={}):
-    from math import sin, cos
+    from math import cos, sin
     m = params.get('m', 4.)      # kg, system mass
     J = params.get('J', 0.0475)  # kg m^2, system inertia
     r = params.get('r', 0.25)    # m, thrust offset
@@ -1526,7 +1608,6 @@ def secord_update(t, x, u, params={}):
     """Second order system dynamics"""
     omega0 = params.get('omega0', 1.)
     zeta = params.get('zeta', 0.5)
-    u = np.array(u, ndmin=1)
     return np.array([
         x[1],
         -2 * zeta * omega0 * x[1] - omega0*omega0 * x[0] + u[0]
@@ -1538,6 +1619,27 @@ def secord_output(t, x, u, params={}):
     return np.array([x[0]])
 
 
+def test_interconnect_name():
+    g = ct.LinearIOSystem(ct.ss(-1,1,1,0),
+                          inputs=['u'],
+                          outputs=['y'],
+                          name='g')
+    k = ct.LinearIOSystem(ct.ss(0,10,2,0),
+                          inputs=['e'],
+                          outputs=['z'],
+                          name='k')
+    h = ct.interconnect([g,k],
+                            inputs=['u','e'],
+                            outputs=['y','z'])
+    assert re.match(r'sys\[\d+\]', h.name), f"Interconnect default name does not match 'sys[]' pattern, got '{h.name}'"
+
+    h = ct.interconnect([g,k],
+                            inputs=['u','e'],
+                            outputs=['y','z'],
+                            name='ic_system')
+    assert h.name == 'ic_system', f"Interconnect name excpected 'ic_system', got '{h.name}'"
+
+
 def test_interconnect_unused_input():
     # test that warnings about unused inputs are reported, or not,
     # as required
@@ -1555,13 +1657,18 @@ def test_interconnect_unused_input():
                           outputs=['u'],
                           name='k')
 
-    with pytest.warns(UserWarning, match=r"Unused input\(s\) in InterconnectedSystem"):
+    with pytest.warns(
+            UserWarning, match=r"Unused input\(s\) in InterconnectedSystem"):
         h = ct.interconnect([g,s,k],
                             inputs=['r'],
                             outputs=['y'])
 
-    with pytest.warns(None) as record:
+    with warnings.catch_warnings():
         # no warning if output explicitly ignored, various argument forms
+        warnings.simplefilter("error")
+        # strip out matrix warnings
+        warnings.filterwarnings("ignore", "the matrix subclass",
+                                category=PendingDeprecationWarning)
         h = ct.interconnect([g,s,k],
                             inputs=['r'],
                             outputs=['y'],
@@ -1576,23 +1683,20 @@ def test_interconnect_unused_input():
         h = ct.interconnect([g,s,k],
                             connections=False)
 
-        #https://docs.pytest.org/en/6.2.x/warnings.html#recwarn
-        for r in record:
-            # strip out matrix warnings
-            if re.match(r'.*matrix subclass', str(r.message)):
-                continue
-            print(r.message)
-            pytest.fail(f'Unexpected warning: {r.message}')
-
-
     # warn if explicity ignored input in fact used
-    with pytest.warns(UserWarning, match=r"Input\(s\) specified as ignored is \(are\) used:") as record:
+    with pytest.warns(
+            UserWarning,
+            match=r"Input\(s\) specified as ignored is \(are\) used:") \
+            as record:
         h = ct.interconnect([g,s,k],
                             inputs=['r'],
                             outputs=['y'],
                             ignore_inputs=['u','n'])
 
-    with pytest.warns(UserWarning, match=r"Input\(s\) specified as ignored is \(are\) used:") as record:
+    with pytest.warns(
+            UserWarning,
+            match=r"Input\(s\) specified as ignored is \(are\) used:") \
+            as record:
         h = ct.interconnect([g,s,k],
                             inputs=['r'],
                             outputs=['y'],
@@ -1623,14 +1727,20 @@ def test_interconnect_unused_output():
                           outputs=['u'],
                           name='k')
 
-    with pytest.warns(UserWarning, match=r"Unused output\(s\) in InterconnectedSystem:") as record:
+    with pytest.warns(
+            UserWarning,
+            match=r"Unused output\(s\) in InterconnectedSystem:") as record:
         h = ct.interconnect([g,s,k],
                             inputs=['r'],
                             outputs=['y'])
 
 
     # no warning if output explicitly ignored
-    with pytest.warns(None) as record:
+    with warnings.catch_warnings():
+        warnings.simplefilter("error")
+        # strip out matrix warnings
+        warnings.filterwarnings("ignore", "the matrix subclass",
+                                category=PendingDeprecationWarning)
         h = ct.interconnect([g,s,k],
                             inputs=['r'],
                             outputs=['y'],
@@ -1645,22 +1755,18 @@ def test_interconnect_unused_output():
         h = ct.interconnect([g,s,k],
                             connections=False)
 
-        #https://docs.pytest.org/en/6.2.x/warnings.html#recwarn
-        for r in record:
-            # strip out matrix warnings
-            if re.match(r'.*matrix subclass', str(r.message)):
-                continue
-            print(r.message)
-            pytest.fail(f'Unexpected warning: {r.message}')
-
     # warn if explicity ignored output in fact used
-    with pytest.warns(UserWarning, match=r"Output\(s\) specified as ignored is \(are\) used:"):
+    with pytest.warns(
+            UserWarning,
+            match=r"Output\(s\) specified as ignored is \(are\) used:"):
         h = ct.interconnect([g,s,k],
                             inputs=['r'],
                             outputs=['y'],
                             ignore_outputs=['dy','u'])
 
-    with pytest.warns(UserWarning, match=r"Output\(s\) specified as ignored is \(are\) used:"):
+    with pytest.warns(
+            UserWarning,
+            match=r"Output\(s\) specified as ignored is \(are\) used:"):
         h = ct.interconnect([g,s,k],
                             inputs=['r'],
                             outputs=['y'],
@@ -1672,3 +1778,272 @@ def test_interconnect_unused_output():
                             inputs=['r'],
                             outputs=['y'],
                             ignore_outputs=['v'])
+
+
+def test_interconnect_add_unused():
+    P = ct.ss(
+        [[-1]], [[1, -1]], [[-1], [1]], 0,
+        inputs=['u1', 'u2'], outputs=['y1','y2'], name='g')
+    S = ct.summing_junction(inputs=['r','-y1'], outputs=['e'], name='s')
+    C = ct.ss(0, 10, 2, 0, inputs=['e'], outputs=['u1'], name='k')
+
+    # Try a normal interconnection
+    G1 = ct.interconnect(
+        [P, S, C], inputs=['r', 'u2'], outputs=['y1', 'y2'])
+
+    # Same system, but using add_unused
+    G2 = ct.interconnect(
+        [P, S, C], inputs=['r'], outputs=['y1'], add_unused=True)
+    assert G2.input_labels == G1.input_labels
+    assert G2.input_offset == G1.input_offset
+    assert G2.output_labels == G1.output_labels
+    assert G2.output_offset == G1.output_offset
+
+    # Ignore one of the inputs
+    G3 = ct.interconnect(
+        [P, S, C], inputs=['r'], outputs=['y1'], add_unused=True,
+        ignore_inputs=['u2'])
+    assert G3.input_labels == G1.input_labels[0:1]
+    assert G3.output_labels == G1.output_labels
+    assert G3.output_offset == G1.output_offset
+
+    # Ignore one of the outputs
+    G4 = ct.interconnect(
+        [P, S, C], inputs=['r'], outputs=['y1'], add_unused=True,
+        ignore_outputs=['y2'])
+    assert G4.input_labels == G1.input_labels
+    assert G4.input_offset == G1.input_offset
+    assert G4.output_labels == G1.output_labels[0:1]
+
+
+def test_input_output_broadcasting():
+    # Create a system, time vector, and noisy input
+    sys = ct.rss(6, 2, 3)
+    T = np.linspace(0, 10, 10)
+    U = np.zeros((sys.ninputs, T.size))
+    U[0, :] = np.sin(T)
+    U[1, :] = np.zeros_like(U[1, :])
+    U[2, :] = np.ones_like(U[2, :])
+    X0 = np.array([1, 2])
+    P0 = np.array([[3.11, 3.12], [3.21, 3.3]])
+
+    # Simulate the system with nominal input to establish baseline
+    resp_base = ct.input_output_response(
+        sys, T, U, np.hstack([X0, P0.reshape(-1)]))
+
+    # Split up the inputs into two pieces
+    resp_inp1 = ct.input_output_response(sys, T, [U[:1], U[1:]], [X0, P0])
+    np.testing.assert_equal(resp_base.states, resp_inp1.states)
+
+    # Specify two of the inputs as constants
+    resp_inp2 = ct.input_output_response(sys, T, [U[0], 0, 1], [X0, P0])
+    np.testing.assert_equal(resp_base.states, resp_inp2.states)
+
+    # Specify two of the inputs as constant vector
+    resp_inp3 = ct.input_output_response(sys, T, [U[0], [0, 1]], [X0, P0])
+    np.testing.assert_equal(resp_base.states, resp_inp3.states)
+
+    # Specify only some of the initial conditions
+    resp_init = ct.input_output_response(sys, T, [U[0], [0, 1]], [X0, 0])
+    resp_cov0 = ct.input_output_response(sys, T, U, [X0, P0 * 0])
+    np.testing.assert_equal(resp_cov0.states, resp_init.states)
+
+    # Specify only some of the initial conditions
+    with pytest.warns(UserWarning, match="initial state too short; padding"):
+        resp_short = ct.input_output_response(sys, T, [U[0], [0, 1]], [X0, 1])
+
+    # Make sure that inconsistent settings don't work
+    with pytest.raises(ValueError, match="inconsistent"):
+        resp_bad = ct.input_output_response(
+            sys, T, (U[0, :], U[:2, :-1]), [X0, P0])
+
+@pytest.mark.parametrize("nstates, ninputs, noutputs", [
+    [2, 1, 1],
+    [4, 2, 3],
+    [0, 1, 1],                  # static function
+    [0, 3, 2],                  # static function
+])
+def test_nonuniform_timepts(nstates, noutputs, ninputs):
+    """Test non-uniform time points for simulations"""
+    if nstates:
+        sys = ct.rss(nstates, noutputs, ninputs)
+    else:
+        sys = ct.ss(
+            [], np.zeros((0, ninputs)), np.zeros((noutputs, 0)),
+            np.random.rand(noutputs, ninputs))
+
+    # Start with a uniform set of times
+    unifpts = [0, 1, 2, 3, 4,  5,  6,  7,  8,  9, 10]
+    uniform = np.outer(
+        np.ones(ninputs), [1, 2, 3, 2, 1, -1, -3, -5, -7, -3,  1])
+    t_unif, y_unif = ct.input_output_response(
+        sys, unifpts, uniform, squeeze=False)
+
+    # Create a non-uniform set of inputs
+    noufpts = [0, 2, 4,  8, 10]
+    nonunif = np.outer(np.ones(ninputs), [1, 3, 1, -7,  1])
+    t_nouf, y_nouf = ct.input_output_response(
+        sys, noufpts, nonunif, squeeze=False)
+
+    # Make sure the outputs agree at common times
+    np.testing.assert_almost_equal(y_unif[:, noufpts], y_nouf, decimal=6)
+
+    # Resimulate using a new set of evaluation points
+    t_even, y_even = ct.input_output_response(
+        sys, noufpts, nonunif, t_eval=unifpts, squeeze=False)
+    np.testing.assert_almost_equal(y_unif, y_even, decimal=6)
+
+
+def test_ss_nonlinear():
+    """Test ss() for creating nonlinear systems"""
+    secord = ct.ss(secord_update, secord_output, inputs='u', outputs='y',
+                   states = ['x1', 'x2'], name='secord')
+    assert secord.name == 'secord'
+    assert secord.input_labels == ['u']
+    assert secord.output_labels == ['y']
+    assert secord.state_labels == ['x1', 'x2']
+
+    # Make sure we get the same answer for simulations
+    T = np.linspace(0, 10, 100)
+    U = np.sin(T)
+    X0 = np.array([1, -1])
+    secord_nlio = ct.NonlinearIOSystem(
+        secord_update, secord_output, inputs=1, outputs=1, states=2)
+    ss_response = ct.input_output_response(secord, T, U, X0)
+    io_response = ct.input_output_response(secord_nlio, T, U, X0)
+    np.testing.assert_almost_equal(ss_response.time, io_response.time)
+    np.testing.assert_almost_equal(ss_response.inputs, io_response.inputs)
+    np.testing.assert_almost_equal(ss_response.outputs, io_response.outputs)
+
+    # Make sure that optional keywords are allowed
+    secord = ct.ss(secord_update, secord_output, dt=True)
+    assert ct.isdtime(secord)
+
+    # Make sure that state space keywords are flagged
+    with pytest.raises(TypeError, match="unrecognized keyword"):
+        ct.ss(secord_update, remove_useless_states=True)
+
+
+def test_rss():
+    # Basic call, with no arguments
+    sys = ct.rss()
+    assert sys.ninputs == 1
+    assert sys.noutputs == 1
+    assert sys.nstates == 1
+    assert sys.dt == 0
+    assert np.all(np.real(sys.poles()) < 0)
+
+    # Set the timebase explicitly
+    sys = ct.rss(inputs=2, outputs=3, states=4, dt=None, name='sys')
+    assert sys.name == 'sys'
+    assert sys.ninputs == 2
+    assert sys.noutputs == 3
+    assert sys.nstates == 4
+    assert sys.dt == None
+    assert np.all(np.real(sys.poles()) < 0)
+
+    # Discrete time
+    sys = ct.rss(inputs=['a', 'b'], outputs=1, states=1, dt=True)
+    assert sys.ninputs == 2
+    assert sys.input_labels == ['a', 'b']
+    assert sys.noutputs == 1
+    assert sys.nstates == 1
+    assert sys.dt == True
+    assert np.all(np.abs(sys.poles()) < 1)
+
+    # Call drss directly
+    sys = ct.drss(inputs=['a', 'b'], outputs=1, states=1, dt=True)
+    assert sys.ninputs == 2
+    assert sys.input_labels == ['a', 'b']
+    assert sys.noutputs == 1
+    assert sys.nstates == 1
+    assert sys.dt == True
+    assert np.all(np.abs(sys.poles()) < 1)
+
+    with pytest.raises(ValueError, match="continuous timebase"):
+        sys = ct.drss(2, 1, 1, dt=0)
+
+    with pytest.warns(UserWarning, match="may be interpreted as continuous"):
+        sys = ct.drss(2, 1, 1, dt=None)
+        assert np.all(np.abs(sys.poles()) < 1)
+
+
+def eqpt_rhs(t, x, u, params):
+    return np.array([x[0]/2 + u[0], x[0] - x[1]**2 + u[1], x[1] - x[2]])
+
+def eqpt_out(t, x, u, params):
+    return np.array([x[0], x[1] + u[1]])
+
+@pytest.mark.parametrize(
+    "x0, ix, u0, iu, y0, iy, dx0, idx, dt, x_expect, u_expect", [
+        # Equilibrium points with input given
+        (0, None, 0, None, None, None, None, None, 0, [0, 0, 0], [0, 0]),
+        (0, None, 0, None, None, None, None, None, None, [0, 0, 0], [0, 0]),
+        ([0.9, 0.9, 0.9], None, [-1, 0], None, None, None, None, None, 0,
+         [2, sqrt(2), sqrt(2)], [-1, 0]),
+        ([0.9, -0.9, 0.9], None, [-1, 0], None, None, None, None, None, 0,
+         [2, -sqrt(2), -sqrt(2)], [-1, 0]),     # same input, different eqpt
+        (0, None, 0, None, None, None, None, None, 1, [0, 0, 0], [0, 0]), #DT
+        (0, None, [-1, 0], None, None, None, None, None, 1, None, None),  #DT
+        ([0, -0.1, 0], None, [0, -0.25], None, None, None, None, None, 1, #DT
+         [0, -0.5, -0.25], [0, -0.25]),
+
+        # Equilibrium points with output given
+        ([0.9, 0.9, 0.9], None, [-0.9, 0], None, [2, sqrt(2)], None, None,
+         None, 0, [2, sqrt(2), sqrt(2)], [-1, 0]),
+        (0, None, [0, -0.25], None, [0, -0.75], None, None, None, 1,      #DT
+         [0, -0.5, -0.25], [0, -0.25]),
+
+        # Equilibrium points with mixture of inputs and outputs given
+        ([0.9, 0.9, 0.9], None, [-1, 0], [0], [2, sqrt(2)], [1], None,
+         None, 0, [2, sqrt(2), sqrt(2)], [-1, 0]),
+        (0, None, [0, -0.22], [0], [0, -0.75], [1], None, None, 1,        #DT
+         [0, -0.5, -0.25], [0, -0.25]),
+    ])
+
+def test_find_eqpt(x0, ix, u0, iu, y0, iy, dx0, idx, dt, x_expect, u_expect):
+    sys = ct.NonlinearIOSystem(
+        eqpt_rhs, eqpt_out, dt=dt, states=3, inputs=2, outputs=2)
+
+    xeq, ueq = ct.find_eqpt(
+        sys, x0, u0, y0, ix=ix, iu=iu, iy=iy, dx0=dx0, idx=idx)
+
+    # If no equilibrium points, skip remaining tests
+    if x_expect is None:
+        assert xeq is None
+        assert ueq is None
+        return
+
+    # Make sure we are at an appropriate equilibrium point
+    if dt is None or dt == 0:
+        # Continuous time system
+        np.testing.assert_allclose(eqpt_rhs(0, xeq, ueq, {}), 0, atol=1e-6)
+        if y0 is not None:
+            y0 = np.array(y0)
+            iy = np.s_[:] if iy is None else np.array(iy)
+            np.testing.assert_allclose(
+                eqpt_out(0, xeq, ueq, {})[iy], y0[iy], atol=1e-6)
+
+    else:
+        # Discrete time system
+        np.testing.assert_allclose(eqpt_rhs(0, xeq, ueq, {}), xeq, atol=1e-6)
+        if y0 is not None:
+            y0 = np.array(y0)
+            iy = np.s_[:] if iy is None else np.array(iy)
+            np.testing.assert_allclose(
+                eqpt_out(0, xeq, ueq, {})[iy], y0[iy], atol=1e-6)
+
+    # Check that we got the expected result as well
+    np.testing.assert_allclose(np.array(xeq), x_expect, atol=1e-6)
+    np.testing.assert_allclose(np.array(ueq), u_expect, atol=1e-6)
+
+def test_iosys_sample():
+    csys = ct.rss(2, 1, 1)
+    dsys = csys.sample(0.1)
+    assert isinstance(dsys, ct.LinearIOSystem)
+    assert dsys.dt == 0.1
+
+    csys = ct.rss(2, 1, 1)
+    dsys = ct.sample_system(csys, 0.1)
+    assert isinstance(dsys, ct.LinearIOSystem)
+    assert dsys.dt == 0.1
