@@ -8,14 +8,13 @@ import pytest
 import operator
 
 import control as ct
-from control.statesp import StateSpace, _convert_to_statespace, rss
-from control.xferfcn import TransferFunction, _convert_to_transfer_function, \
-    ss2tf
-from control.lti import evalfr
-from control.tests.conftest import slycotonly, nopython2, matrixfilter
-from control.lti import isctime, isdtime
-from control.dtime import sample_system
-from control.config import defaults
+from control import StateSpace, TransferFunction, rss, evalfr
+from control import ss, ss2tf, tf, tf2ss, zpk
+from control import isctime, isdtime, sample_system
+from control import defaults, reset_defaults, set_defaults
+from control.statesp import _convert_to_statespace
+from control.xferfcn import _convert_to_transfer_function
+from control.tests.conftest import slycotonly, matrixfilter
 
 
 class TestXferFcn:
@@ -42,7 +41,7 @@ class TestXferFcn:
             TransferFunction([1])
 
         # Too many arguments
-        with pytest.raises(ValueError):
+        with pytest.raises(TypeError):
             TransferFunction(1, 2, 3, 4)
 
         # Different numbers of elements in numerator rows
@@ -88,18 +87,19 @@ class TestXferFcn:
             TransferFunction([[[1.], [2., 3.]], [[-1., 4.], [3., 2.]]],
                              [[[1., 0.], [0.]], [[0., 0.], [2.]]])
 
+    @pytest.mark.skip("outdated test")
     def test_constructor_nodt(self):
         """Test the constructor when an object without dt is passed"""
         sysin = TransferFunction([[[0., 1.], [2., 3.]]],
                                  [[[5., 2.], [3., 0.]]])
-        del sysin.dt
+        del sysin.dt            # this doesn't make sense and now breaks
         sys = TransferFunction(sysin)
         assert sys.dt == defaults['control.default_dt']
 
         # test for static gain
         sysin = TransferFunction([[[2.], [3.]]],
                                  [[[1.], [.1]]])
-        del sysin.dt
+        del sysin.dt            # this doesn't make sense and now breaks
         sys = TransferFunction(sysin)
         assert sys.dt is None
 
@@ -450,7 +450,6 @@ class TestXferFcn:
             np.testing.assert_allclose(sys.evalfr(omega), resp, atol=1e-3)
 
 
-    @nopython2
     def test_call_dtime(self):
         sys = TransferFunction([1., 3., 5], [1., 6., 2., -1], 0.1)
         np.testing.assert_array_almost_equal(sys(1j), -0.5 - 0.5j)
@@ -599,7 +598,7 @@ class TestXferFcn:
         sys = TransferFunction(
             [[[1.], [1.]], [[1.], [1.]]],
             [[[1., 2.], [1., 3.]], [[1., 4., 4.], [1., 9., 14.]]])
-        p = sys.pole()
+        p = sys.poles()
 
         np.testing.assert_array_almost_equal(p, [-2., -2., -7., -3., -2.])
 
@@ -607,14 +606,14 @@ class TestXferFcn:
         sys2 = TransferFunction(
             [[[1., 2., 3., 4.], [1.]], [[1.], [1.]]],
             [[[1., 2.], [1., 3.]], [[1., 4., 4.], [1., 9., 14.]]])
-        p2 = sys2.pole()
+        p2 = sys2.poles()
 
         np.testing.assert_array_almost_equal(p2, [-2., -2., -7., -3., -2.])
 
     def test_double_cancelling_poles_siso(self):
 
         H = TransferFunction([1, 1], [1, 2, 1])
-        p = H.pole()
+        p = H.poles()
         np.testing.assert_array_almost_equal(p, [-1, -1])
 
     # Tests for TransferFunction.feedback
@@ -743,8 +742,7 @@ class TestXferFcn:
         "matarrayin",
         [pytest.param(np.array,
                       id="arrayin",
-                      marks=[nopython2,
-                             pytest.mark.skip(".__matmul__ not implemented")]),
+                      marks=[pytest.mark.skip(".__matmul__ not implemented")]),
          pytest.param(np.matrix,
                       id="matrixin",
                       marks=matrixfilter)],
@@ -909,6 +907,128 @@ class TestXferFcn:
         assert isinstance(str(sys), str)
         assert isinstance(sys._repr_latex_(), str)
 
+    @pytest.mark.parametrize(
+        "zeros, poles, gain, output",
+        [([0], [-1], 1,
+          '\n'
+          '  s\n'
+          '-----\n'
+          's + 1\n'),
+         ([-1], [-1], 1,
+          '\n'
+          's + 1\n'
+          '-----\n'
+          's + 1\n'),
+         ([-1], [1], 1,
+          '\n'
+          's + 1\n'
+          '-----\n'
+          's - 1\n'),
+         ([1], [-1], 1,
+          '\n'
+          's - 1\n'
+          '-----\n'
+          's + 1\n'),
+         ([-1], [-1], 2,
+          '\n'
+          '2 (s + 1)\n'
+          '---------\n'
+          '  s + 1\n'),
+         ([-1], [-1], 0,
+          '\n'
+          '0\n'
+          '-\n'
+          '1\n'),
+         ([-1], [1j, -1j], 1,
+          '\n'
+          '      s + 1\n'
+          '-----------------\n'
+          '(s - 1j) (s + 1j)\n'),
+         ([4j, -4j], [2j, -2j], 2,
+          '\n'
+          '2 (s - 4j) (s + 4j)\n'
+          '-------------------\n'
+          ' (s - 2j) (s + 2j)\n'),
+         ([1j, -1j], [-1, -4], 2,
+          '\n'
+          '2 (s - 1j) (s + 1j)\n'
+          '-------------------\n'
+          '  (s + 1) (s + 4)\n'),
+         ([1], [-1 + 1j, -1 - 1j], 1,
+          '\n'
+          '          s - 1\n'
+          '-------------------------\n'
+          '(s + (1-1j)) (s + (1+1j))\n'),
+         ([1], [1 + 1j, 1 - 1j], 1,
+          '\n'
+          '          s - 1\n'
+          '-------------------------\n'
+          '(s - (1+1j)) (s - (1-1j))\n'),
+         ])
+    def test_printing_zpk(self, zeros, poles, gain, output):
+        """Test _tf_polynomial_to_string for constant systems"""
+        G = zpk(zeros, poles, gain, display_format='zpk')
+        res = str(G)
+        assert res == output
+
+    @pytest.mark.parametrize(
+        "zeros, poles, gain, format, output",
+        [([1], [1 + 1j, 1 - 1j], 1, ".2f",
+          '\n'
+          '                1.00\n'
+          '-------------------------------------\n'
+          '(s + (1.00-1.41j)) (s + (1.00+1.41j))\n'),
+         ([1], [1 + 1j, 1 - 1j], 1, ".3f",
+          '\n'
+           '                  1.000\n'
+           '-----------------------------------------\n'
+           '(s + (1.000-1.414j)) (s + (1.000+1.414j))\n'),
+         ([1], [1 + 1j, 1 - 1j], 1, ".6g",
+          '\n'
+          '                  1\n'
+          '-------------------------------------\n'
+          '(s + (1-1.41421j)) (s + (1+1.41421j))\n')
+         ])
+    def test_printing_zpk_format(self, zeros, poles, gain, format, output):
+        """Test _tf_polynomial_to_string for constant systems"""
+        G = tf([1], [1,2,3], display_format='zpk')
+
+        set_defaults('xferfcn', floating_point_format=format)
+        res = str(G)
+        reset_defaults()
+
+        assert res == output
+
+    @pytest.mark.parametrize(
+        "num, den, output",
+        [([[[11], [21]], [[12], [22]]],
+         [[[1, -3, 2], [1, 1, -6]], [[1, 0, 1], [1, -1, -20]]],
+         ('\n'
+          'Input 1 to output 1:\n'
+          '      11\n'
+          '---------------\n'
+          '(s - 2) (s - 1)\n'
+          '\n'
+          'Input 1 to output 2:\n'
+          '       12\n'
+          '-----------------\n'
+          '(s - 1j) (s + 1j)\n'
+          '\n'
+          'Input 2 to output 1:\n'
+          '      21\n'
+          '---------------\n'
+          '(s - 2) (s + 3)\n'
+          '\n'
+          'Input 2 to output 2:\n'
+          '      22\n'
+          '---------------\n'
+          '(s - 5) (s + 4)\n'))])
+    def test_printing_zpk_mimo(self, num, den, output):
+        """Test _tf_polynomial_to_string for constant systems"""
+        G = tf(num, den, display_format='zpk')
+        res = str(G)
+        assert res == output
+
     @slycotonly
     def test_size_mismatch(self):
         """Test size mismacht"""
@@ -991,6 +1111,37 @@ class TestXferFcn:
                 np.testing.assert_array_almost_equal(H.den[p][m], H2.den[p][m])
             assert H.dt == H2.dt
 
+    def test_sample_named_signals(self):
+        sysc = ct.TransferFunction(1.1, (1, 2), inputs='u', outputs='y')
+
+        # Full form of the call
+        sysd = sysc.sample(0.1, name='sampled')
+        assert sysd.name == 'sampled'
+        assert sysd.find_input('u') == 0
+        assert sysd.find_output('y') == 0
+
+        # If we copy signal names w/out a system name, append '$sampled'
+        sysd = sysc.sample(0.1)
+        assert sysd.name == sysc.name + '$sampled'
+
+        # If copy is False, signal names should not be copied
+        sysd_nocopy = sysc.sample(0.1, copy_names=False)
+        assert sysd_nocopy.find_input('u') is None
+        assert sysd_nocopy.find_output('y') is None
+
+        # if signal names are provided, they should override those of sysc
+        sysd_newnames = sysc.sample(0.1, inputs='v', outputs='x')
+        assert sysd_newnames.find_input('v') == 0
+        assert sysd_newnames.find_input('u') is None
+        assert sysd_newnames.find_output('x') == 0
+        assert sysd_newnames.find_output('y') is None
+        # test just one name
+        sysd_newnames = sysc.sample(0.1, inputs='v')
+        assert sysd_newnames.find_input('v') == 0
+        assert sysd_newnames.find_input('u') is None
+        assert sysd_newnames.find_output('y') == 0
+        assert sysd_newnames.find_output('x') is None
+
 
 class TestLTIConverter:
     """Test returnScipySignalLTI method"""
@@ -1046,3 +1197,78 @@ def test_xferfcn_ndarray_precedence(op, tf, arr):
     # Apply the operator to the array and transfer function
     result = op(arr, tf)
     assert isinstance(result, ct.TransferFunction)
+
+
+@pytest.mark.parametrize(
+    "zeros, poles, gain, args, kwargs", [
+        ([], [-1], 1, [], {}),
+        ([1, 2], [-1, -2, -3], 5, [], {}),
+        ([1, 2], [-1, -2, -3], 5, [], {'name': "sys"}),
+        ([1, 2], [-1, -2, -3], 5, [], {'inputs': ["in"], 'outputs': ["out"]}),
+        ([1, 2], [-1, -2, -3], 5, [0.1], {}),
+        (np.array([1, 2]), np.array([-1, -2, -3]), 5, [], {}),
+])
+def test_zpk(zeros, poles, gain, args, kwargs):
+    # Create the transfer function
+    sys = ct.zpk(zeros, poles, gain, *args, **kwargs)
+
+    # Make sure the poles and zeros match
+    np.testing.assert_equal(sys.zeros().sort(), zeros.sort())
+    np.testing.assert_equal(sys.poles().sort(), poles.sort())
+
+    # Check to make sure the gain is OK
+    np.testing.assert_almost_equal(
+        gain, sys(0) * np.prod(-sys.poles()) / np.prod(-sys.zeros()))
+
+    # Check time base
+    if args:
+        assert sys.dt == args[0]
+
+    # Check inputs, outputs, name
+    input_labels = kwargs.get('inputs', [])
+    for i, label in enumerate(input_labels):
+        assert sys.input_labels[i] == label
+
+    output_labels = kwargs.get('outputs', [])
+    for i, label in enumerate(output_labels):
+        assert sys.output_labels[i] == label
+
+    if kwargs.get('name'):
+        assert sys.name == kwargs.get('name')
+
+@pytest.mark.parametrize("create, args, kwargs, convert", [
+    (StateSpace, ([-1], [1], [1], [0]), {}, ss2tf),
+    (StateSpace, ([-1], [1], [1], [0]), {}, ss),
+    (StateSpace, ([-1], [1], [1], [0]), {}, tf),
+    (StateSpace, ([-1], [1], [1], [0]), dict(inputs='i', outputs='o'), ss2tf),
+    (StateSpace, ([-1], [1], [1], [0]), dict(inputs=1, outputs=1), ss2tf),
+    (StateSpace, ([-1], [1], [1], [0]), dict(inputs='i', outputs='o'), ss),
+    (StateSpace, ([-1], [1], [1], [0]), dict(inputs='i', outputs='o'), tf),
+    (TransferFunction, ([1], [1, 1]), {}, tf2ss),
+    (TransferFunction, ([1], [1, 1]), {}, tf),
+    (TransferFunction, ([1], [1, 1]), {}, ss),
+    (TransferFunction, ([1], [1, 1]), dict(inputs='i', outputs='o'), tf2ss),
+    (TransferFunction, ([1], [1, 1]), dict(inputs=1, outputs=1), tf2ss),
+    (TransferFunction, ([1], [1, 1]), dict(inputs='i', outputs='o'), tf),
+    (TransferFunction, ([1], [1, 1]), dict(inputs='i', outputs='o'), ss),
+])
+def test_copy_names(create, args, kwargs, convert):
+    # Convert a system with no renaming
+    sys = create(*args, **kwargs, name='sys')
+    cpy = convert(sys)
+
+    assert cpy.input_labels == sys.input_labels
+    assert cpy.input_labels == sys.input_labels
+    if cpy.nstates is not None and sys.nstates is not None:
+        assert cpy.state_labels == sys.state_labels
+
+    # Make sure that names aren't the same if system changed type
+    if not isinstance(cpy, create):
+        assert cpy.name == sys.name + '$converted'
+    else:
+        assert cpy.name == sys.name
+
+    # Relabel inputs and outputs
+    cpy = convert(sys, inputs='myin', outputs='myout')
+    assert cpy.input_labels == ['myin']
+    assert cpy.output_labels == ['myout']
