@@ -79,7 +79,8 @@ from scipy.linalg import eig, eigvals, matrix_balance, norm
 from copy import copy
 
 from . import config
-from .lti import isctime, isdtime
+from .exception import pandas_check
+from .namedio import isctime, isdtime
 from .statesp import StateSpace, _convert_to_statespace, _mimo2simo, _mimo2siso
 from .xferfcn import TransferFunction
 
@@ -87,7 +88,7 @@ __all__ = ['forced_response', 'step_response', 'step_info',
            'initial_response', 'impulse_response', 'TimeResponseData']
 
 
-class TimeResponseData():
+class TimeResponseData:
     """A class for returning time responses.
 
     This class maintains and manipulates the data corresponding to the
@@ -462,7 +463,7 @@ class TimeResponseData():
         # Update any keywords that we were passed
         response.transpose = kwargs.pop('transpose', self.transpose)
         response.squeeze = kwargs.pop('squeeze', self.squeeze)
-        response.return_x = kwargs.pop('return_x', self.squeeze)
+        response.return_x = kwargs.pop('return_x', self.return_x)
 
         # Check for new labels
         input_labels = kwargs.pop('input_labels', None)
@@ -480,9 +481,9 @@ class TimeResponseData():
             response.state_labels = _process_labels(
                 state_labels, "state", response.nstates)
 
-        # Make sure no unknown keywords were passed
-        if len(kwargs) != 0:
-            raise ValueError("Unknown parameter(s) %s" % kwargs)
+        # Make sure there were no extraneous keywords
+        if kwargs:
+            raise TypeError("unrecognized keywords: ", str(kwargs))
 
         return response
 
@@ -638,6 +639,23 @@ class TimeResponseData():
     def __len__(self):
         return 3 if self.return_x else 2
 
+    # Convert to pandas
+    def to_pandas(self):
+        if not pandas_check():
+            raise ImportError("pandas not installed")
+        import pandas
+
+        # Create a dict for setting up the data frame
+        data = {'time': self.time}
+        data.update(
+            {name: self.u[i] for i, name in enumerate(self.input_labels)})
+        data.update(
+            {name: self.y[i] for i, name in enumerate(self.output_labels)})
+        data.update(
+            {name: self.x[i] for i, name in enumerate(self.state_labels)})
+
+        return pandas.DataFrame(data)
+
 
 # Process signal labels
 def _process_labels(labels, signal, length):
@@ -676,7 +694,10 @@ def _process_labels(labels, signal, length):
             raise ValueError("Name dictionary for %s is incomplete" % signal)
 
     # Convert labels to a list
-    labels = list(labels)
+    if isinstance(labels, str):
+        labels = [labels]
+    else:
+        labels = list(labels)
 
     # Make sure the signal list is the right length and type
     if len(labels) != length:
@@ -799,7 +820,7 @@ def _check_convert_array(in_obj, legal_shapes, err_msg_start, squeeze=False,
 # Forced response of a linear system
 def forced_response(sys, T=None, U=0., X0=0., transpose=False,
                     interpolate=False, return_x=None, squeeze=None):
-    """Simulate the output of a linear system.
+    """Compute the output of a linear system given the input.
 
     As a convenience for parameters `U`, `X0`:
     Numbers (scalars) are converted to constant arrays with the correct shape.
@@ -898,7 +919,9 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False,
 
     Examples
     --------
-    >>> T, yout, xout = forced_response(sys, T, u, X0)
+    >>> G = ct.rss(4)
+    >>> T = np.linspace(0, 10)
+    >>> T, yout = ct.forced_response(G, T=T)
 
     See :ref:`time-series-convention` and
     :ref:`package-configuration-parameters`.
@@ -954,11 +977,6 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False,
             dt = 1. if sys.dt in [True, None] else sys.dt
             T = np.array(range(n_steps)) * dt
         else:
-            # Make sure the input vector and time vector have same length
-            if (U.ndim == 1 and U.shape[0] != T.shape[0]) or \
-                    (U.ndim > 1 and U.shape[1] != T.shape[0]):
-                raise ValueError('Parameter ``T`` must have same elements as'
-                                 ' the number of columns in input array ``U``')
             if U.ndim == 0:
                 U = np.full((n_inputs, T.shape[0]), U)
     else:
@@ -1096,6 +1114,8 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False,
 
     return TimeResponseData(
         tout, yout, xout, U, issiso=sys.issiso(),
+        output_labels=sys.output_labels, input_labels=sys.input_labels,
+        state_labels=sys.state_labels,
         transpose=transpose, return_x=return_x, squeeze=squeeze)
 
 
@@ -1315,7 +1335,8 @@ def step_response(sys, T=None, X0=0., input=None, output=None, T_num=None,
 
     Examples
     --------
-    >>> T, yout = step_response(sys, T, X0)
+    >>> G = ct.rss(4)
+    >>> T, yout = ct.step_response(G)
 
     """
     # Create the time and input vectors
@@ -1358,8 +1379,16 @@ def step_response(sys, T=None, X0=0., input=None, output=None, T_num=None,
     # Figure out if the system is SISO or not
     issiso = sys.issiso() or (input is not None and output is not None)
 
+    # Select only the given input and output, if any
+    input_labels = sys.input_labels if input is None \
+        else sys.input_labels[input]
+    output_labels = sys.output_labels if output is None \
+        else sys.output_labels[output]
+
     return TimeResponseData(
         response.time, yout, xout, uout, issiso=issiso,
+        output_labels=output_labels, input_labels=input_labels,
+        state_labels=sys.state_labels,
         transpose=transpose, return_x=return_x, squeeze=squeeze)
 
 
@@ -1371,7 +1400,7 @@ def step_info(sysdata, T=None, T_num=None, yfinal=None,
     Parameters
     ----------
     sysdata : StateSpace or TransferFunction or array_like
-        The system data. Either LTI system to similate (StateSpace,
+        The system data. Either LTI system to simulate (StateSpace,
         TransferFunction), or a time series of step response data.
     T : array_like or float, optional
         Time vector, or simulation time duration if a number (time vector is
@@ -1427,9 +1456,8 @@ def step_info(sysdata, T=None, T_num=None, yfinal=None,
 
     Examples
     --------
-    >>> from control import step_info, TransferFunction
-    >>> sys = TransferFunction([-1, 1], [1, 1, 1])
-    >>> S = step_info(sys)
+    >>> sys = ct.TransferFunction([-1, 1], [1, 1, 1])
+    >>> S = ct.step_info(sys)
     >>> for k in S:
     ...     print(f"{k}: {S[k]:3.4}")
     ...
@@ -1447,15 +1475,14 @@ def step_info(sysdata, T=None, T_num=None, yfinal=None,
     characteristics for the second input and specify a 5% error until the
     signal is considered settled.
 
-    >>> from numpy import sqrt
-    >>> from control import step_info, StateSpace
-    >>> sys = StateSpace([[-1., -1.],
+    >>> from math import sqrt
+    >>> sys = ct.StateSpace([[-1., -1.],
     ...                   [1., 0.]],
     ...                  [[-1./sqrt(2.), 1./sqrt(2.)],
     ...                   [0, 0]],
     ...                  [[sqrt(2.), -sqrt(2.)]],
     ...                  [[0, 0]])
-    >>> S = step_info(sys, T=10., SettlingTimeThreshold=0.05)
+    >>> S = ct.step_info(sys, T=10., SettlingTimeThreshold=0.05)
     >>> for k, v in S[0][1].items():
     ...     print(f"{k}: {float(v):3.4}")
     RiseTime: 1.212
@@ -1589,7 +1616,7 @@ def step_info(sysdata, T=None, T_num=None, yfinal=None,
 def initial_response(sys, T=None, X0=0., input=0, output=None, T_num=None,
                      transpose=False, return_x=False, squeeze=None):
     # pylint: disable=W0622
-    """Initial condition response of a linear system
+    """Compute the initial condition response for a linear system.
 
     If the system has multiple outputs (MIMO), optionally, one output
     may be selected. If no selection is made for the output, all
@@ -1673,7 +1700,8 @@ def initial_response(sys, T=None, X0=0., input=0, output=None, T_num=None,
 
     Examples
     --------
-    >>> T, yout = initial_response(sys, T, X0)
+    >>> G = ct.rss(4)
+    >>> T, yout = ct.initial_response(G)
 
     """
     squeeze, sys = _get_ss_simo(sys, input, output, squeeze=squeeze)
@@ -1689,9 +1717,15 @@ def initial_response(sys, T=None, X0=0., input=0, output=None, T_num=None,
     # Figure out if the system is SISO or not
     issiso = sys.issiso() or (input is not None and output is not None)
 
+    # Select only the given output, if any
+    output_labels = sys.output_labels if output is None \
+        else sys.output_labels[0]
+
     # Store the response without an input
     return TimeResponseData(
         response.t, response.y, response.x, None, issiso=issiso,
+        output_labels=output_labels, input_labels=None,
+        state_labels=sys.state_labels,
         transpose=transpose, return_x=return_x, squeeze=squeeze)
 
 
@@ -1783,11 +1817,13 @@ def impulse_response(sys, T=None, X0=0., input=None, output=None, T_num=None,
     -----
     This function uses the `forced_response` function to compute the time
     response. For continuous time systems, the initial condition is altered to
-    account for the initial impulse.
+    account for the initial impulse. For discrete-time aystems, the impulse is
+    sized so that it has unit area.
 
     Examples
     --------
-    >>> T, yout = impulse_response(sys, T, X0)
+    >>> G = ct.rss(4)
+    >>> T, yout = ct.impulse_response(G)
 
     """
     # Convert to state space so that we can simulate
@@ -1852,8 +1888,16 @@ def impulse_response(sys, T=None, X0=0., input=None, output=None, T_num=None,
     # Figure out if the system is SISO or not
     issiso = sys.issiso() or (input is not None and output is not None)
 
+    # Select only the given input and output, if any
+    input_labels = sys.input_labels if input is None \
+        else sys.input_labels[input]
+    output_labels = sys.output_labels if output is None \
+        else sys.output_labels[output]
+
     return TimeResponseData(
         response.time, yout, xout, uout, issiso=issiso,
+        output_labels=output_labels, input_labels=input_labels,
+        state_labels=sys.state_labels,
         transpose=transpose, return_x=return_x, squeeze=squeeze)
 
 
